# << Haru Free PDF Library 2.0.0 >> -- character_map.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.
#
# usage tclsh character_map.tcl <encoding-name> <low-range-from>
#               <low-range-to> <high-range-from> <high-range-to>
#  ex. character_map 90ms-RKSJ-V 0x80 0x
lappend auto_path ..
package require Hpdf

proc draw_page {pdf page title_font font h_byte l_byte} {
    set PAGE_WIDTH  420
    set CELL_HEIGHT  20
    set CELL_WIDTH   20

    set l_byte  [expr {int($l_byte / 16) * 16}]
    set h_count [expr {16 - ($l_byte / 16)}]
    set page_height [expr {40 + 40 + ($h_count + 1) * $CELL_HEIGHT}]

    $page setHeight $page_height
    $page setWidth  $PAGE_WIDTH

    $page setFontAndSize $title_font 10

    set ypos [expr {$h_count + 1}]
    while {1} {
        set y [expr {$ypos * $CELL_HEIGHT + 40}]

        $page moveTo 40  $y
        $page lineTo 380 $y
        $page stroke

        if {$ypos < $h_count} {
            set buf [format "%X" [expr {16 - $ypos -1}]]

            set w [$page textWidth $buf]
            $page beginText
            $page moveTextPos [expr {40 + (20.0 - $w) / 2.0}] [expr {$y + 5}]
            $page showText $buf
            $page endText
        }

        if {$ypos == 0} {break}
        incr ypos -1
    }

    for {set xpos 0} {$xpos <= 17} {incr xpos} {
        set y [expr {($h_count + 1) * $CELL_HEIGHT + 40}]
        set x [expr {$xpos * $CELL_WIDTH + 40}]

        $page moveTo $x 40
        $page lineTo $x $y
        $page stroke

        if {$xpos > 0 && $xpos <= 16} {
            set buf [format "%X" [expr {$xpos -1}]]

            set w [$page textWidth $buf]
            $page beginText
            $page moveTextPos \
                [expr {$x + (20 - $w) / 2}] \
                [expr {$h_count * $CELL_HEIGHT + 45}]
            $page showText $buf
            $page endText
        }
    }
    
    $page setFontAndSize $font 15
    
    set ypos $h_count
    while {1} {
        set y [expr {($ypos - 1) * $CELL_HEIGHT + 45}]

        for {set xpos 0} {$xpos < 16} {incr xpos} {
            set x [expr {$xpos * $CELL_WIDTH + 40 + $CELL_WIDTH}]
            set buf [binary format c2 \
                    [list $h_byte [expr {(16 - $ypos) * 16 + $xpos}] ]]
            set w [$page textWidth $buf]
            if {$w > 0} {
                $page beginText
                $page moveTextPos [expr {$x + (20 - $w) / 2.0}] $y
                $page showText $buf
                $page endText
            }
        }
        if {$ypos == 0} {break}

        incr ypos -1
    }
}

proc main {argv} {
    set defenc [encoding system]
    set pdf [hpdf::new]

    $pdf setPageMode outline
    $pdf setCompressionMode all
    $pdf setPagesConfiguration 10


    $pdf useJPEncodings
    $pdf useJPFonts
    $pdf useKREncodings
    $pdf useKRFonts
    $pdf useCNSEncodings
    $pdf useCNSFonts
    $pdf useCNTEncodings
    $pdf useCNTFonts

    set title_font [$pdf getFont Helvetica]

    set encoder [$pdf getEncoder [lindex $argv 0]]
    if {[$encoder getType] ne "double_byte"} {
        hpdf::free $pdf
        error [format "error: %s is not cmap-encoder\n" [lindex $argv 0]]
    }
    
    set font [$pdf getFont [lindex $argv 1] [lindex $argv 0]]

    set min_l 255
    set min_h 256
    set max_l 0
    set max_h 0

    for {set i 0} {$i <= 255} {incr i} {
        set flg($i) 0
    }

    for {set i 0} {$i <= 255} {incr i} {
        for {set j 20} {$j <= 255} {incr j} {
            set code [expr {$i * 256 + $j}]
            set buf [binary format c2 [list $i $j]]
            set btype [$encoder getByteType $buf 0]
            set unicode [$encoder getUnicode $code]

            if {$btype eq "lead" && $unicode != 0x25A1} {
                if {$min_l > $j} {set min_l $j}
                if {$max_l < $j} {set max_l $j}
                if {$min_h > $i} {set min_h $i}
                if {$max_h < $i} {set max_h $i}
                set flg($i) 1
            }
        }
    }

    puts [format "min_h=%04X max_h=%04X min_l=%04X max_l=%04X\n" \
            $min_h $max_h $min_l $max_l]

    # create outline root.
    set root [$pdf createOutline "" [lindex $argv 0] ""]
    $root setOpened yes

    for {set i 0} {$i <= 255} {incr i} {
        if {$flg($i)} {
            set page [$pdf addPage]
            set title_font [$pdf getFont "Helvetica" ""]
            set buf [format "0x%04X-0x%04X" \
                [expr {($i * 256 + $min_l)}] \
                [expr {($i * 256 + $max_l)}]]
            set outline [$pdf createOutline $root $buf ""]
            set dst [$page createDestination]
            $outline setDestination $dst

            draw_page $pdf $page $title_font $font $i $min_l

            set buf [format "%s (%s) 0x%04X-0x%04X" \
                [lindex $argv 0] \
                [lindex $argv 1] \
                [expr {int($i * 256 + $min_l)}] \
                [expr {int($i * 256 + $max_l)}]]
            $page setFontAndSize $title_font 10
            $page beginText
            $page moveTextPos 40 [expr {[$page getHeight] - 35}]
            $page showText $buf
            $page endText
        }
    }

    encoding system $defenc
    $pdf saveToFile [file rootname [info script]].pdf
    hpdf::free $pdf
}

main $argv
