# << Haru Free PDF Library 2.0.0 >> -- encoding_list.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.

lappend auto_path ..
package require Hpdf


set PAGE_WIDTH  420
set PAGE_HEIGHT 400
set CELL_WIDTH  20
set CELL_HEIGHT 20
set CELL_HEADER 10


proc draw_graph {page} {
    # Draw 16 X 15 cells

    # Draw vertical lines.
    $page setLineWidth 0.5

    for {set i 0} {$i <= 17} {incr i} {
        set x [expr {$i * $::CELL_WIDTH +40}]
        $page moveTo $x [expr {$::PAGE_HEIGHT - 60}]
        $page lineTo $x 40
        $page stroke

        if {$i > 0 && $i <= 16} {
            $page beginText
            $page moveTextPos [expr {$x + 5}] [expr {$::PAGE_HEIGHT - 75}]
            $page showText [format "%X" [expr {$i - 1}]]
            $page endText
        }
    }

    # Draw horizontal lines.
    for {set i 0} {$i <= 15} {incr i} {
        set y [expr {$i * $::CELL_HEIGHT + 40}]

        $page moveTo 40 $y
        $page lineTo [expr {$::PAGE_WIDTH - 40}] $y
        $page stroke

        if {$i < 14} {
            $page beginText
            $page moveTextPos 45 [expr {$y + 5}]
            $page showText [format "%X" [expr {15 - $i}]]
            $page endText
        }
    }
}

proc draw_fonts {page} {
    $page beginText

    # Draw all character from 0x20 to 0xFF to the canvas.
    for {set i 1} {$i < 17} {incr i} {
        for {set j 1} {$j < 17} {incr j} {
            set y [expr {$::PAGE_HEIGHT - 55 - (($i - 1) * $::CELL_HEIGHT)}]
            set x [expr {$j * $::CELL_WIDTH + 50}]
            
            set buf [expr {($i - 1) * 16 + ($j - 1)}]
            if {$buf >= 32} {
                set t [binary format i $buf]
                set tw [$page textWidth $t]
                set d [expr {$x - $tw / 2}]
                $page textOut $d $y $t
            }
        }
    }

    $page endText
}

proc main {} {
    set pdf [hpdf::new]
    set encodings {
        StandardEncoding
        MacRomanEncoding
        WinAnsiEncoding
        ISO8859-2
        ISO8859-3
        ISO8859-4
        ISO8859-5
        ISO8859-9
        ISO8859-10
        ISO8859-13
        ISO8859-14
        ISO8859-15
        ISO8859-16
        CP1250
        CP1251
        CP1252
        CP1254
        CP1257
        KOI8-R
        Symbol-Set
        ZapfDingbats-Set
    }

    # set compression mode
    $pdf setCompressionMode all

    # Set page mode to use outlines.
    $pdf setPageMode outline

    # get default font
    set font [$pdf getFont "Helvetica"]

    # load font object
    set font_name [$pdf loadType1FontFromFile \
        "type1/a010013l.afm" "type1/a010013l.pfb"]

    # create outline root.
    set root [$pdf createOutline "" "Encoding list" ""]
    $root setOpened yes

    foreach enc $encodings {
        set page [$pdf addPage]
        $page setWidth $::PAGE_WIDTH
        $page setHeight $::PAGE_HEIGHT
        
        set outline [$pdf createOutline $root $enc ""]

        set dst [$page createDestination]
        $dst setXYZ 0 [$page getHeight] 1

        $outline setDestination $dst

        $page setFontAndSize $font 15
        draw_graph $page

        $page beginText
        $page setFontAndSize $font 20
        $page moveTextPos 40 [expr {$::PAGE_HEIGHT - 50}]
        $page showText $enc
        $page showText " Encoding"
        $page endText
        
        switch -exact -- $enc {
            "Symbol-Set" {
            set font2 [$pdf getFont "Symbol"]
            }
            "ZapfDingbats-Set" {
            set font2 [$pdf getFont "ZapfDingbats"]
            }
            default {
            set font2 [$pdf getFont $font_name $enc]
            }
        }

        $page setFontAndSize $font2 14
        draw_fonts $page
    }

    $pdf saveToFile [file rootname [info script]].pdf
    hpdf::free $pdf
}
main
