# << Haru Free PDF Library 2.0.0 >> -- ext_gstate_demo.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.

lappend auto_path ..
package require Hpdf

proc draw_circles {page description x y} {
    $page setLineWidth 1.0
    $page setRGBStroke  0.0 0.0 0.0

    $page setRGBFill 1.0 0.0 0.0
    $page circle [expr {$x + 40}] [expr {$y + 40}] 40
    $page closePathFillStroke

    $page setRGBFill  0.0 1.0 0.0
    $page circle [expr {$x + 100}] [expr {$y + 40}] 40
    $page closePathFillStroke

    $page setRGBFill 0.0 0.0 1.0
    $page circle [expr {$x + 70}] [expr {$y + 74.64}] 40
    $page closePathFillStroke

    $page setRGBFill 0.0 0.0 0.0
    $page beginText
    $page textOut [expr {$x + 0.0}] [expr {$y + 130.0}] $description
    $page endText
}

proc main {} {
    set PAGE_WIDTH 600
    set PAGE_HEIGHT 900

    set pdf [hpdf::new]

    set hfont [$pdf getFont "Helvetica-Bold"]

    set page [$pdf addPage]

    $page setFontAndSize $hfont 10

    $page setHeight $PAGE_HEIGHT
    $page setWidth $PAGE_WIDTH

    # normal
    $page gSave
    draw_circles $page "normal" 40.0 [expr {$PAGE_HEIGHT - 170}]
    $page gRestore

    # transparency (0.8)
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setAlphaFill 0.8
    $gstate setAlphaStroke 0.8
    $page setExtGState $gstate
    draw_circles $page "alpha fill = 0.8" 230.0 [expr {$PAGE_HEIGHT - 170}]
    $page gRestore

    # transparency (0.4)
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setAlphaFill 0.4
    $page setExtGState $gstate
    draw_circles $page "alpha fill = 0.4" 420.0 [expr {$PAGE_HEIGHT - 170}]
    $page gRestore

    # blend-mode=HPDF_BM_MULTIPLY
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setBlendMode multiply
    $page setExtGState $gstate
    draw_circles $page "HPDF_BM_MULTIPLY" 40.0 [expr {$PAGE_HEIGHT - 340}]
    $page gRestore

    # blend-mode=HPDF_BM_SCREEN
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setBlendMode screen
    $page setExtGState $gstate
    draw_circles $page "HPDF_BM_SCREEN" 230.0 [expr {$PAGE_HEIGHT - 340}]
    $page gRestore

    # blend-mode=HPDF_BM_OVERLAY
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setBlendMode overlay
    $page setExtGState $gstate
    draw_circles $page "HPDF_BM_OVERLAY" 420.0 [expr {$PAGE_HEIGHT - 340}]
    $page gRestore

    # blend-mode=HPDF_BM_DARKEN
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setBlendMode darken
    $page setExtGState $gstate
    draw_circles $page "HPDF_BM_DARKEN" 40.0 [expr {$PAGE_HEIGHT - 510}]
    $page gRestore

    # blend-mode=HPDF_BM_LIGHTEN
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setBlendMode lighten
    $page setExtGState $gstate
    draw_circles $page "HPDF_BM_LIGHTEN" 230.0 [expr {$PAGE_HEIGHT - 510}]
    $page gRestore

    # blend-mode=HPDF_BM_COLOR_DODGE
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setBlendMode color_dodge
    $page setExtGState $gstate
    draw_circles $page "HPDF_BM_COLOR_DODGE" 420.0 [expr {$PAGE_HEIGHT - 510}]
    $page gRestore

    # blend-mode=HPDF_BM_COLOR_BUM
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setBlendMode color_bum
    $page setExtGState $gstate
    draw_circles $page "HPDF_BM_COLOR_BUM" 40.0 [expr {$PAGE_HEIGHT - 680}]
    $page gRestore

    # blend-mode=HPDF_BM_HARD_LIGHT
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setBlendMode hard_light
    $page setExtGState $gstate
    draw_circles $page "HPDF_BM_HARD_LIGHT" 230.0 [expr {$PAGE_HEIGHT - 680}]
    $page gRestore

    # blend-mode=HPDF_BM_SOFT_LIGHT
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setBlendMode soft_light
    $page setExtGState $gstate
    draw_circles $page "HPDF_BM_SOFT_LIGHT" 420.0 [expr {$PAGE_HEIGHT - 680}]
    $page gRestore

    # blend-mode=HPDF_BM_DIFFERENCE
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setBlendMode difference
    $page setExtGState $gstate
    draw_circles $page "HPDF_BM_DIFFERENCE" 40.0 [expr {$PAGE_HEIGHT - 850}]
    $page gRestore


    # blend-mode=HPDF_BM_EXCLUSHON
    $page gSave
    set gstate [$pdf createExtGState]
    $gstate setBlendMode exclushon
    $page setExtGState $gstate
    draw_circles $page "HPDF_BM_EXCLUSHON" 230.0 [expr {$PAGE_HEIGHT - 850}]
    $page gRestore

    $pdf saveToFile [file rootname [info script]].pdf
    
    hpdf::free $pdf
}

main


