# << Haru Free PDF Library 2.0.0 >> -- grid_sheet.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.

lappend auto_path ..
package require Hpdf

proc print_grid {pdf page} {
    set height [$page getHeight]
    set width  [$page getWidth]
    set font   [$pdf getFont "Helvetica"]

    $page setFontAndSize $font 5
    $page setGrayFill 0.5
    $page setGrayStroke 0.8

    # Draw horizontal lines
    set y 0
    while {$y < $height} {
        if {($y % 10) == 0} {
            $page setLineWidth 0.5
        } else {
            if {[$page getLineWidth] != 0.25} {
                $page setLineWidth 0.25
            }
        }

        $page moveTo 0 $y
        $page lineTo $width $y
        $page stroke 

        if {($y % 10) == 0 && $y > 0} {
            $page setGrayStroke 0.5

            $page moveTo 0 $y
            $page lineTo 5 $y
            $page stroke 

            $page setGrayStroke 0.8
        }

        incr y 5
    }


    # Draw virtical lines
    set x 0
    while {$x < $width} {
        if {($x % 10) == 0} {
            $page setLineWidth 0.5
        } else {
            if {[$page getLineWidth] != 0.25} {
                $page setLineWidth 0.25
            }
        }

        $page moveTo $x 0
        $page lineTo $x $height
        $page stroke 

        if {($x % 50) == 0 && $x > 0} {
            $page setGrayStroke 0.5

            $page moveTo $x 0
            $page lineTo $x 5
            $page stroke 

            $page moveTo $x $height
            $page lineTo $x [expr {$height - 5}]
            $page stroke 

            $page setGrayStroke 0.8
        }

        incr x 5;
    }

    # Draw horizontal text
    set y 0
    while {$y < $height} {
        if {($y % 10) == 0 && $y > 0} {

            $page beginText
            $page moveTextPos 5 [expr {$y - 2}]

            $page showText [format "%u" $y]
            $page endText
        }

        incr y 5
    }


    # Draw virtical text
    set x 0
    while {$x < $width} {
        if {($x % 50) == 0 && $x > 0} {
            $page beginText
            $page moveTextPos $x 5

            $page showText [format "%u" $x]
            $page endText

            $page beginText
            $page moveTextPos $x [expr {$height - 10}]
            $page showText [format "%u" $x]
            $page endText
        }

        incr x 5
    }

    $page setGrayFill 0
    $page setGrayStroke 0
}


proc main {} {
    set pdf [hpdf::new]
    set page [$pdf addPage]
    $page setHeight 600
    $page setWidth 400

    print_grid $pdf $page

    $pdf saveToFile [file rootname [info script]].pdf
    hpdf::free $pdf
}

main
