# << Haru Free PDF Library 2.0.0 >> -- image_demo.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.

lappend auto_path ..
package require Hpdf

proc show_description {page x y text} {
    $page moveTo $x [expr {$y - 10}]
    $page lineTo $x [expr {$y + 10}]
    $page moveTo [expr {$x - 10}] $y
    $page lineTo [expr {$x + 10}] $y
    $page stroke 

    $page setFontAndSize [$page getCurrentFont] 8
    $page setRGBFill 0 0 0

    $page beginText

    set buf [format "(x=%d,y=%d)" [expr {int($x)}] [expr {int($y)}]]
    $page moveTextPos \
        [expr {$x - [$page textWidth $buf] - 5}]\
        [expr {$y - 10}]
    $page showText $buf
    $page endText

    $page beginText
    $page moveTextPos [expr {$x - 20}] [expr {$y - 25}]
    $page showText $text
    $page endText
}

proc main {} {
    set pdf [hpdf::new]

    $pdf setCompressionMode all

    # create default-font
    set font [$pdf getFont "Helvetica"]

    # add a new page object.
    set page [$pdf addPage]
    
    $page setWidth 550
    $page setHeight 500

    set dst [$page createDestination]
    $dst setXYZ 0 [$page getHeight] 1
    $pdf setOpenAction $dst

    $page beginText
    $page setFontAndSize $font 20
    $page moveTextPos 220 [expr {[$page getHeight] - 70}]
    $page showText "ImageDemo"
    $page endText



    # load image file.
    set image [$pdf loadPngImageFromFile "pngsuite/basn3p02.png"]

    # image1 is masked by image2.
    set image1 [$pdf loadPngImageFromFile "pngsuite/basn3p02.png"]

    # image2 is a mask image.
    set image2 [$pdf loadPngImageFromFile "pngsuite/basn0g01.png"]

    # image3 is a RGB-color image. 
    # we use this image for color-mask demo.
    set image3 [$pdf loadPngImageFromFile "pngsuite/maskimage.png"]

    set iw [$image getWidth]
    set ih [$image getHeight]

    $page setLineWidth 0.5

    set x 100;
    set y [expr {[$page getHeight] - 150}]

    # Draw image to the canvas. (normal-mode with actual size.)
    $page drawImage $image $x $y $iw $ih

    show_description $page $x $y "Actual Size"

    incr x 150

    # Scalling image (X direction)
    $page drawImage $image $x $y [expr {$iw * 1.5}] $ih

    show_description $page $x $y "Scalling image (X direction)"

    incr x 150

    # Scalling image (Y direction).
    $page drawImage $image $x $y $iw [expr {$ih * 1.5}]
    show_description $page $x $y "Scalling image (Y direction)"

    set x 100
    set y [expr {$y -120}]

    # Skewing image.
    set angle1 10
    set angle2 20
    set rad1 [expr {$angle1 / 180.0 * 3.141592}]
    set rad2 [expr {$angle2 / 180.0 * 3.141592}]

    $page gSave

    $page concat $iw \
        [expr {tan($rad1) * $iw}] \
        [expr {tan($rad2) * $ih}] \
        $ih $x $y

    $page executeXObject $image
    $page gRestore

    show_description $page $x $y "Skewing image"

    incr x 150

    # Rotating image
    set angle 30 ;# rotation of 30 degrees.
    set rad [expr {$angle / 180.0 * 3.141592}];# Calcurate the radian value.

    $page gSave

    $page concat [expr {$iw * cos($rad)}] \
                 [expr {$iw * sin($rad)}] \
                 [expr {$ih * -sin($rad)}] \
                 [expr {$ih * cos($rad)}] \
                 $x $y

    $page executeXObject $image
    $page gRestore

    show_description $page $x $y "Rotating image"

    incr x 150

    # draw masked image.

    # Set image2 to the mask image of image1
    $image1 setMaskImage $image2

    $page setRGBFill 0 0 0
    $page beginText
    $page moveTextPos [expr {$x - 6}] [expr {$y + 14}]
    $page showText "MASKMASK"
    $page endText

    $page drawImage $image1 \
        [expr {$x - 3}] [expr {$y - 3}] \
        [expr {$iw + 6}] [expr {$ih + 6}]

    show_description $page $x $y "masked image"

    set x 100
    set y [expr {$y -120}]

    # color mask.
    $page setRGBFill 0 0 0
    $page beginText
    $page moveTextPos [expr {$x - 6}] [expr {$y + 14}]
    $page showText "MASKMASK"
    $page endText

    $image3 setColorMask 0 255 0 0 0 255
    $page drawImage $image3 $x $y $iw $ih

    show_description $page $x $y "Color Mask"

    # save the document to a file
    $pdf saveToFile [file rootname [info script]].pdf

    # clean up 
    hpdf::free $pdf

    return 0;
}

main
