# << Haru Free PDF Library 2.0.0 >> -- jpfont_demo.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.

lappend auto_path ..
package require Hpdf

encoding system cp932
set PAGE_HEIGHT 210


if {[catch {open "mbtext/sjis.txt" r} fp]} {
    puts "error: cannot open 'mbtext/sjis.txt"
    exit
}
fconfigure $fp -encoding shiftjis
set samp_text [read $fp]
close $fp

set pdf [hpdf::new]

$pdf setCompressionMode all
$pdf useJPEncodings
$pdf useJPFonts

set fontlist {
    MS-Mincyo               90ms-RKSJ-H
    MS-Mincyo,Bold          90ms-RKSJ-H
    MS-Mincyo,Italic        90ms-RKSJ-H
    MS-Mincyo,BoldItalic    90ms-RKSJ-H
    MS-PMincyo              90msp-RKSJ-H
    MS-PMincyo,Bold         90msp-RKSJ-H
    MS-PMincyo,Italic       90msp-RKSJ-H
    MS-PMincyo,BoldItalic   90msp-RKSJ-H
    MS-Gothic               90ms-RKSJ-H
    MS-Gothic,Bold          90ms-RKSJ-H
    MS-Gothic,Italic        90ms-RKSJ-H
    MS-Gothic,BoldItalic    90ms-RKSJ-H
    MS-PGothic              90msp-RKSJ-H
    MS-PGothic,Bold         90msp-RKSJ-H
    MS-PGothic,Italic       90msp-RKSJ-H
    MS-PGothic,BoldItalic   90msp-RKSJ-H
}

# Set page mode to use outlines.
$pdf setPageMode outline

# create outline root.
set root [$pdf createOutline "" "JP font demo" ""]
$root setOpened yes

foreach {name enc} $fontlist {
    set detail_font [$pdf getFont $name $enc]

    # add a new page object.
    set page [$pdf addPage]

    # create outline entry
    set outline [$pdf createOutline $root \
                [$detail_font getFontName] ""]
    set dst [$page createDestination]
    $outline setDestination $dst

    set title_font [$pdf getFont "Helvetica"]
    $page setFontAndSize $title_font 10

    $page beginText

    # move the position of the text to top of the page.
    $page moveTextPos 10 190
    $page showText [$detail_font getFontName]

    $page setFontAndSize $detail_font 15
    $page moveTextPos 10 -20
    $page showText "abcdefghijklmnopqrstuvwxyz"
    $page moveTextPos 0 -20
    $page showText "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    $page moveTextPos 0 -20
    $page showText "1234567890"
    $page moveTextPos 0 -20

    $page setFontAndSize $detail_font 10
    $page showText $samp_text
    $page moveTextPos 0 -18

    $page setFontAndSize $detail_font 16
    $page showText $samp_text
    $page moveTextPos 0 -27

    $page setFontAndSize $detail_font 23
    $page showText $samp_text
    $page moveTextPos 0 -36

    $page setFontAndSize $detail_font 30
    $page showText $samp_text

    foreach {px py} [$page getCurrentTextPos] {break}

    # finish to print text.
    $page endText

    $page setLineWidth 0.5

    set x_pos 20

    for {set j 0} {$j <= [string length $samp_text]/2} {incr j} {
        $page moveTo $x_pos [expr {$py - 10}]
        $page lineTo $x_pos [expr {$py - 12}]
        $page stroke
        incr x_pos 30
    }

    $page setWidth [expr {$px + 20}]
    $page setHeight $PAGE_HEIGHT

    $page moveTo 10 [expr {$PAGE_HEIGHT - 25}]
    $page lineTo [expr {$px + 10}] [expr {$PAGE_HEIGHT - 25}]
    $page stroke

    $page moveTo 10 [expr {$PAGE_HEIGHT - 85}]
    $page lineTo [expr {$px + 10}] [expr {$PAGE_HEIGHT - 85}]
    $page stroke 

    $page moveTo 10 [expr {$py - 12}]
    $page lineTo [expr {$px + 10}] [expr {$py - 12}]
    $page stroke 
}


# save the document to a file
$pdf saveToFile [file rootname [info script]].pdf

# clean up 
hpdf::free $pdf

exit
