# << Haru Free PDF Library 2.0.0 >> -- line_demo.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.

lappend auto_path ..
package require Hpdf

proc draw_line {page x y text} {
    $page beginText
    $page moveTextPos $x [expr {$y - 10}]
    $page showText $text
    $page endText

    $page moveTo $x [expr {$y - 15}]
    $page lineTo [expr {$x + 220}] [expr {$y - 15}]
    $page stroke
}

proc draw_line2 {page x y text} {
    $page beginText
    $page moveTextPos $x $y
    $page showText $text
    $page endText

    $page moveTo [expr {$x + 30}]  [expr {$y - 25}]
    $page lineTo [expr {$x + 160}] [expr {$y - 25}]
    $page stroke
}

proc draw_rect {page x y text} {
    $page beginText
    $page moveTextPos $x [expr {$y - 10}]
    $page showText $text
    $page endText

    $page rectangle $x [expr {$y - 40}] 220 25
}

set pdf [hpdf::new]

$pdf setCompressionMode all

set font [$pdf getFont Helvetica]

set page [$pdf addPage]

# print the lines of the pages.
$page setLineWidth 1
set x [expr {[$page getWidth] - 100}]
set y [expr {[$page getHeight] - 110}]
$page rectangle 50 50 $x $y
$page stroke

# print the title of the page (with positioning center).
$page setFontAndSize $font 24.0
set tw [$page textWidth "Line Demo"]
$page beginText

set x [expr {([$page getWidth] - $tw) / 2}]
set y [expr {[$page getHeight] - 50}]
$page moveTextPos $x $y

$page showText "Line Demo"
$page endText

$page setFontAndSize $font 10

# draw verious widths of lines.
$page setLineWidth 0
draw_line $page 60 770 "line width = 0"

$page setLineWidth 1.0
draw_line $page 60 740 "line width = 1.0"

$page setLineWidth 2.0
draw_line $page 60 710 "line width = 2.0"

# line dash pattern
$page setLineWidth 1.0

$page setDash {3} 2
draw_line $page 60 680 {dash_ptn=[3], phase=1 -- 2 on, 3 off, 3 on...}

$page setDash {7 3} 2
draw_line $page 60 650 {dash_ptn=[7, 3], phase=2 -- 5 on, 3 off, 7 on...}

$page setDash {8 7 2 7} 0
draw_line $page 60 620 {dash_ptn=[8, 7, 2, 7], phase=0}

$page setDash {} 0

$page setLineWidth 30
$page setRGBStroke 0.0 0.5 0.0

# line cap style
$page setLineCap butt_end
draw_line2 $page 60 570 "HPDF_BUTT_END"

$page setLineCap round_end
draw_line2 $page 60 505 "HPDF_ROUND_END"

$page setLineCap projecting_scuare_end
draw_line2 $page 60 440 "HPDF_PROJECTING_SCUARE_END"

# line join style
$page setLineWidth 30
$page setRGBStroke 0.0 0.0 0.5

$page setLineJoin miter_join
$page moveTo 120 300
$page lineTo 160 340
$page lineTo 200 300
$page stroke

$page beginText
$page moveTextPos 60 360
$page showText miter_join
$page endText

$page setLineJoin round_join
$page moveTo 120 195
$page lineTo 160 235
$page lineTo 200 195
$page stroke

$page beginText
$page moveTextPos 60 255
$page showText "HPDF_ROUND_JOIN"
$page endText

$page setLineJoin bevel_join
$page moveTo 120 90
$page lineTo 160 130
$page lineTo 200 90
$page stroke

$page beginText
$page moveTextPos 60 150
$page showText "HPDF_BEVEL_JOIN"
$page endText

# draw rectangle
$page setLineWidth 2
$page setRGBStroke 0 0 0
$page setRGBFill 0.75 0.0 0.0

draw_rect $page 300 770 "stroke"
$page stroke

draw_rect $page 300 720 "Fill"
$page fill

draw_rect $page 300 670 "Fill then stroke"
$page fillStroke

# clip rect
$page gSave  ;# Save the current graphic state
draw_rect $page 300 620 "Clip rectangle"
$page clip
$page stroke
$page setFontAndSize $font 13

$page beginText
$page moveTextPos 290 600
$page setTextLeading 12
$page showText "Clip Clip Clip Clip Clip Clipi Clip Clip Clip"
$page showTextNextLine "Clip Clip Clip Clip Clip Clip Clip Clip Clip"
$page showTextNextLine "Clip Clip Clip Clip Clip Clip Clip Clip Clip"
$page endText
$page gRestore

set x0 330
set y0 440
set x1 430
set y1 530
set x2 480
set y2 470
set x3 480
set y3 90

# curve example(curve_to2)
$page setRGBFill 0 0 0

$page beginText
$page moveTextPos 300 540
$page showText "curve_to2(x1, y1, x2. y2)"
$page endText

$page beginText
$page moveTextPos [expr {$x0 + 5}] [expr {$y0 - 5}]
$page showText "Current point"
$page moveTextPos [expr {$x1 - $x0}] [expr {$y1 - $y0}]
$page showText "(x1, y1)"
$page moveTextPos [expr {$x2 - $x1}] [expr {$y2 - $y1}]
$page showText "(x2, y2)"
$page endText

$page setDash {} 0

$page setLineWidth 0.5
$page moveTo $x1 $y1
$page lineTo $x2 $y2
$page stroke

$page setDash {} 0

$page setLineWidth 1.5

$page moveTo $x0 $y0
$page curveTo2 $x1 $y1 $x2 $y2
$page stroke

# curve example(curve_to3)
incr y0 -150
incr y1 -150
incr y2 -150

$page beginText
$page moveTextPos 300 390
$page showText "curve_to3(x1, y1, x2. y2)"
$page endText

$page beginText
$page moveTextPos [expr {$x0 + 5}] [expr {$y0 - 5}]
$page showText "Current point"
$page moveTextPos [expr {$x1 - $x0}] [expr {$y1 - $y0}]
$page showText "(x1, y1)"
$page moveTextPos [expr {$x2 - $x1}] [expr {$y2 - $y1}]
$page showText "(x2, y2)"
$page endText

$page setDash {3} 1

$page setLineWidth 0.5
$page moveTo $x0 $y0
$page lineTo $x1 $y1
$page stroke

$page setDash {} 0

$page setLineWidth 1.5
$page moveTo $x0 $y0
$page curveTo3 $x1 $y1 $x2 $y2
$page stroke

# curve example(curve_to)
incr y0 -150
incr y1 -160
incr y2 -130
incr x2 +10

$page beginText
$page moveTextPos 300 240
$page showText "curve_to(x1, y1, x2. y2, x3, y3)"
$page endText

$page beginText
$page moveTextPos [expr {$x0 + 5}] [expr {$y0 - 5}]
$page showText "Current point"
$page moveTextPos [expr {$x1 - $x0}] [expr {$y1 - $y0}]
$page showText "(x1, y1)"
$page moveTextPos [expr {$x2 - $x1}] [expr {$y2 - $y1}]
$page showText "(x2, y2)"
$page moveTextPos [expr {$x3 - $x2}] [expr {$y3 - $y2}]
$page showText "(x3, y3)"
$page endText

$page setDash {3} 1

$page setLineWidth 0.5
$page moveTo $x0 $y0
$page lineTo $x1 $y1
$page stroke
$page moveTo $x2 $y2
$page lineTo $x3 $y3
$page stroke

$page setDash {} 0

$page setLineWidth 1.5
$page moveTo $x0 $y0
$page curveTo $x1 $y1 $x2 $y2 $x3 $y3
$page stroke

$pdf saveToFile [file rootname [info script]].pdf
