# << Haru Free PDF Library 2.0.0 >> -- link_annotation.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.

lappend auto_path ..
package require Hpdf


proc print_page {page font page_num} {
    $page setWidth 200
    $page setHeight 200

    $page setFontAndSize $font 20

    $page beginText
    $page moveTextPos 50 150
    set buf [format "Page:%d" $page_num]

    $page showText $buf
    $page endText
}


proc main {} {
    set uri "http://sourceforge.net/projects/libharu"

    set pdf [hpdf::new]

    # create default-font
    set font [$pdf getFont "Helvetica"]

    # create index page
    set index_page [$pdf addPage]
    $index_page setWidth  300
    $index_page setHeight 220

    # Add 7 pages to the document.
    for {set i 0} {$i < 7} {incr i} {
        set page($i) [$pdf addPage]
        print_page $page($i) $font [expr {$i + 1}]
    }

    $index_page beginText
    $index_page setFontAndSize $font 10
    $index_page moveTextPos 15 200
    $index_page showText "Link Annotation Demo"
    $index_page endText

    #
    # Create Link-Annotation object on index page.
    #
    $index_page beginText
    $index_page setFontAndSize $font 8
    $index_page moveTextPos 20 180
    $index_page setTextLeading 23

    # page1 (HPDF_ANNOT_NO_HIGHTLIGHT)
    foreach {tpx tpy} [$index_page getCurrentTextPos] {break}

    $index_page showText "Jump to Page1 (HilightMode=HPDF_ANNOT_NO_HIGHTLIGHT)"

    set rect [list]
    lappend rect [expr {$tpx - 4}]
    lappend rect [expr {$tpy - 4}]
    lappend rect [expr {[lindex [$index_page getCurrentTextPos] 0] + 4}]
    lappend rect [expr {$tpy + 10}]

    $index_page moveToNextLine

    set dst [$page(0) createDestination]

    set annot [$index_page createLinkAnnot $rect $dst]

    $annot setHighlightMode no_hightlight


    # page2 (HPDF_ANNOT_INVERT_BOX)
    foreach {tpx tpy} [$index_page getCurrentTextPos] {break}

    $index_page showText "Jump to Page2 (HilightMode=HPDF_ANNOT_INVERT_BOX)"
    set rect [list]
    lappend rect [expr {$tpx - 4}]
    lappend rect [expr {$tpy - 4}]
    lappend rect [expr {[lindex [$index_page getCurrentTextPos] 0] + 4}]
    lappend rect [expr {$tpy + 10}]

    $index_page moveToNextLine

    set dst [$page(1) createDestination]

    set annot [$index_page createLinkAnnot $rect $dst]

    $annot setHighlightMode invert_box


    # page3 (HPDF_ANNOT_INVERT_BORDER)
    foreach {tpx tpy} [$index_page getCurrentTextPos] {break}

    $index_page showText "Jump to Page3 (HilightMode=HPDF_ANNOT_INVERT_BORDER)"
    set rect [list]
    lappend rect [expr {$tpx - 4}]
    lappend rect [expr {$tpy - 4}]
    lappend rect [expr {[lindex [$index_page getCurrentTextPos] 0] + 4}]
    lappend rect [expr {$tpy + 10}]

    $index_page moveToNextLine

    set dst [$page(2) createDestination]

    set annot [$index_page createLinkAnnot $rect $dst]

    $annot setHighlightMode invert_border


    # page4 (HPDF_ANNOT_DOWN_APPEARANCE)
    foreach {tpx tpy} [$index_page getCurrentTextPos] {break}

    $index_page showText "Jump to Page4 (HilightMode=HPDF_ANNOT_DOWN_APPEARANCE)"
    set rect [list]
    lappend rect [expr {$tpx - 4}]
    lappend rect [expr {$tpy - 4}]
    lappend rect [expr {[lindex [$index_page getCurrentTextPos] 0] + 4}]
    lappend rect [expr {$tpy + 10}]

    $index_page moveToNextLine

    set dst [$page(3) createDestination]

    set annot [$index_page createLinkAnnot $rect $dst]

    $annot setHighlightMode down_appearance


    # page5 (dash border)
    foreach {tpx tpy} [$index_page getCurrentTextPos] {break}

    $index_page showText "Jump to Page5 (dash border)"
    set rect [list]
    lappend rect [expr {$tpx - 4}]
    lappend rect [expr {$tpy - 4}]
    lappend rect [expr {[lindex [$index_page getCurrentTextPos] 0] + 4}]
    lappend rect [expr {$tpy + 10}]

    $index_page moveToNextLine

    set dst [$page(4) createDestination]

    set annot [$index_page createLinkAnnot $rect $dst]

    $annot setBorderStyle 1 3 2


    # page6 (no border)
    foreach {tpx tpy} [$index_page getCurrentTextPos] {break}

    $index_page showText "Jump to Page6 (no border)"
    set rect [list]
    lappend rect [expr {$tpx - 4}]
    lappend rect [expr {$tpy - 4}]
    lappend rect [expr {[lindex [$index_page getCurrentTextPos] 0] + 4}]
    lappend rect [expr {$tpy + 10}]

    $index_page moveToNextLine

    set dst [$page(5) createDestination]

    set annot [$index_page createLinkAnnot $rect $dst]

    $annot setBorderStyle 0 0 0


    # page7 (bold border)
    foreach {tpx tpy} [$index_page getCurrentTextPos] {break}

    $index_page showText "Jump to Page7 (bold border)"
    set rect [list]
    lappend rect [expr {$tpx - 4}]
    lappend rect [expr {$tpy - 4}]
    lappend rect [expr {[lindex [$index_page getCurrentTextPos] 0] + 4}]
    lappend rect [expr {$tpy + 10}]

    $index_page moveToNextLine

    set dst [$page(6) createDestination]

    set annot [$index_page createLinkAnnot $rect $dst]

    $annot setBorderStyle 2 0 0


    # URI link
    foreach {tpx tpy} [$index_page getCurrentTextPos] {break}

    $index_page showText "URI ("
    $index_page showText $uri
    $index_page showText ")"

    set rect [list]
    lappend rect [expr {$tpx - 4}]
    lappend rect [expr {$tpy - 4}]
    lappend rect [expr {[lindex [$index_page getCurrentTextPos] 0] + 4}]
    lappend rect [expr {$tpy + 10}]

    $index_page createURILinkAnnot $rect $uri

    $index_page endText

    # save the document to a file
    $pdf saveToFile [file rootname [info script]].pdf

    # clean up 
    hpdf::free $pdf
}

main
