# << Haru Free PDF Library 2.0.0 >> -- outline_demo.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.

lappend auto_path ..
package require Hpdf


proc print_page {page page_num} {
    $page setWidth 800
    $page setHeight 800

    $page beginText
    $page moveTextPos 30 740
    set buf [format "Page:%d" $page_num]

    $page showText $buf
    $page endText
}


proc main {} {
    set pdf [hpdf::new]

    # create default-font
    set font [$pdf getFont "Helvetica"]

    # Set page mode to use outlines.
    $pdf setPageMode outline

    # Add 3 pages to the document.
    set page(0) [$pdf addPage]
    $page(0) setFontAndSize $font 30
    print_page $page(0) 1

    set page(1) [$pdf addPage]
    $page(1) setFontAndSize $font 30
    print_page $page(1) 2

    set page(2) [$pdf addPage]
    $page(2) setFontAndSize $font 30
    print_page $page(2) 3

    # create outline root.
    set root [$pdf createOutline "" "OutlineRoot" ""]
    $root setOpened yes

    set outline(0) [$pdf createOutline $root "page1" ""]
    set outline(1) [$pdf createOutline $root "page2" ""]

    # create outline with test which is ISO8859-2 encoding
    set enc [encoding system]
    encoding system iso8859-2
    set outline(2) [$pdf createOutline $root \
        "ISO8859-2 text  \xd3\xd4\xd5\xd6\xd7\xd8"\
        [$pdf getEncoder "ISO8859-2"]]
    encoding system $enc
    # create destination objects on each pages
    # and link it to outline items.
    #
    set dst [$page(0) createDestination]
    $dst setXYZ 0 [$page(0) getHeight] 1
    $outline(0) setDestination $dst

    set dst [$page(1) createDestination]
    $dst setXYZ 0 [$page(1) getHeight] 1
    $outline(1) setDestination $dst

    set dst [$page(2) createDestination]
    $dst setXYZ 0 [$page(2) getHeight] 1
    $outline(2) setDestination $dst

    # save the document to a file
    $pdf saveToFile [file rootname [info script]].pdf

    # clean up 
    hpdf::free $pdf
}

main
