# << Haru Free PDF Library 2.0.0 >> -- outline_demo_jp.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.

lappend auto_path ..
package require Hpdf


proc print_page {page page_num} {
    $page setWidth 200
    $page setHeight 300

    $page beginText
    $page moveTextPos 50 250
    set buf [format "Page:%d" $page_num]

    $page showText $buf
    $page endText
}


proc main {} {

    if {[catch {open "mbtext/sjis.txt"} fp]} {
        puts "error: cannot open 'mbtext/sjis.txt'"
    }
    fconfigure $fp -encoding shiftjis
    set samp_txt [read $fp]
    close $fp

    set pdf [hpdf::new]

    # declaration for using Japanese encoding.
    $pdf useJPEncodings

    # create default-font
    set font [$pdf getFont "Helvetica"]

    # Set page mode to use outlines.
    $pdf setPageMode outline

    # Add 3 pages to the document.
    set page(0) [$pdf addPage]
    $page(0) setFontAndSize $font 20
    print_page $page(0) 1

    set page(1) [$pdf addPage]
    $page(1) setFontAndSize $font 20
    print_page $page(1) 2

    set page(2) [$pdf addPage]
    $page(2) setFontAndSize $font 20
    print_page $page(2) 3

    # create outline root.
    set root [$pdf createOutline "" "OutlineRoot" ""]
    $root setOpened yes

    set outline(0) [$pdf createOutline $root "page1" ""]
    set outline(1) [$pdf createOutline $root "page2" ""]

    # create outline with test which is  encoding
    set outline(2) [$pdf createOutline $root $samp_txt \
                   [$pdf getEncoder "90ms-RKSJ-H"]]

    # create destination objects on each pages
    # and link it to outline items.
    set dst [$page(0) createDestination]
    $dst setXYZ 0 [$page(0) getHeight] 1
    $outline(0) setDestination $dst

    set dst [$page(1) createDestination]
    $dst setXYZ 0 [$page(1) getHeight] 1
    $outline(1) setDestination $dst

    set dst [$page(2) createDestination]
    $dst setXYZ 0 [$page(2) getHeight] 1
    $outline(2) setDestination $dst

    # save the document to a file
    $pdf saveToFile [file rootname [info script]].pdf

    # clean up
    hpdf::free $pdf

    return 0
}

main
