# << Haru Free PDF Library 2.0.0 >> -- slide_show_demo.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.

lappend auto_path ..
package require Hpdf

proc print_page {page caption font style prev next} {
    set r [expr {rand()}]
    set g [expr {rand()}]
    set b [expr {rand()}]

    $page setWidth 800
    $page setHeight 600

    $page setRGBFill $r $g $b

    $page rectangle 0 0 800 600
    $page fill

    $page setRGBFill \
        [expr {1.0 - $r}] \
        [expr {1.0 - $g}] \
        [expr {1.0 - $b}]

    $page setFontAndSize $font 30

    $page beginText
    $page setTextMatrix 0.8 0.0 0.0 1.0 0.0 0.0
    $page textOut 50 530 $caption

    $page setTextMatrix 1.0 0.0 0.0 1.0 0.0 0.0
    $page setFontAndSize $font 20
    $page textOut 55 300 \
    "Type \"Ctrl+L\" in order to return from full screen mode."
    $page endText

    $page setSlideShow $style 5.0 1.0

    $page setFontAndSize $font 20

    if {$next ne ""} {
        $page beginText
        $page textOut 680 50 "Next=>"
        $page endText
        
        set rect [list]
        lappend rect 680 ;#left
        lappend rect 50  ;#bottom
        lappend rect 750 ;#right
        lappend rect 70  ;#top
        set dst [$next createDestination]
        $dst setFit
        set annot [$page createLinkAnnot $rect $dst]
        $annot setBorderStyle 0 0 0
        $annot setHighlightMode invert_box
    }

    if {$prev ne ""} {
        $page beginText
        $page textOut 50 50 "<=Prev"
        $page endText
        
        set rect [list]
        lappend rect 50  ;#left
        lappend rect 50  ;#bottom
        lappend rect 110 ;#right
        lappend rect 70  ;#top
        set dst [$prev createDestination]
        $dst setFit
        set annot [$page createLinkAnnot $rect $dst]
        $annot setBorderStyle 0 0 0
        $annot setHighlightMode invert_box
    }
}

proc main {} {
    set pdf [hpdf::new]

    # create default-font
    set font [$pdf getFont "Courier"]

    # Add 17 pages to the document.
    set page(0) [$pdf addPage]
    set page(1) [$pdf addPage]
    set page(2) [$pdf addPage]
    set page(3) [$pdf addPage]
    set page(4) [$pdf addPage]
    set page(5) [$pdf addPage]
    set page(6) [$pdf addPage]
    set page(7) [$pdf addPage]
    set page(8) [$pdf addPage]
    set page(9) [$pdf addPage]
    set page(10) [$pdf addPage]
    set page(11) [$pdf addPage]
    set page(12) [$pdf addPage]
    set page(13) [$pdf addPage]
    set page(14) [$pdf addPage]
    set page(15) [$pdf addPage]
    set page(16) [$pdf addPage]

    print_page $page(0) "HPDF_TS_WIPE_RIGHT" $font \
			wipe_right "" $page(1)
    print_page $page(1) "HPDF_TS_WIPE_UP" $font \
            wipe_up $page(0) $page(2)
    print_page $page(2) "HPDF_TS_WIPE_LEFT" $font \
            wipe_left $page(1) $page(3)
    print_page $page(3) "HPDF_TS_WIPE_DOWN" $font \
            wipe_down $page(2) $page(4)
    print_page $page(4) "HPDF_TS_BARN_DOORS_HORIZONTAL_OUT" $font \
            barn_doors_horizontal_out $page(3) $page(5)
    print_page $page(5) "HPDF_TS_BARN_DOORS_HORIZONTAL_IN" $font \
            barn_doors_horizontal_in $page(4) $page(6)
    print_page $page(6) "HPDF_TS_BARN_DOORS_VERTICAL_OUT" $font \
            barn_doors_vertical_out $page(5) $page(7)
    print_page $page(7) "HPDF_TS_BARN_DOORS_VERTICAL_IN" $font \
            barn_doors_vertical_in $page(6) $page(8)
    print_page $page(8) "HPDF_TS_BOX_OUT" $font \
            box_out $page(7) $page(9)
    print_page $page(9) "HPDF_TS_BOX_IN" $font \
            box_in $page(8) $page(10)
    print_page $page(10) "HPDF_TS_BLINDS_HORIZONTAL" $font \
            blinds_horizontal $page(9) $page(11)
    print_page $page(11) "HPDF_TS_BLINDS_VERTICAL" $font \
            blinds_vertical $page(10) $page(12)
    print_page $page(12) "HPDF_TS_DISSOLVE" $font \
            dissolve $page(11) $page(13)
    print_page $page(13) "HPDF_TS_GLITTER_RIGHT" $font \
            glitter_right $page(12) $page(14)
    print_page $page(14) "HPDF_TS_GLITTER_DOWN" $font \
            glitter_down $page(13) $page(15)
    print_page $page(15) "HPDF_TS_GLITTER_TOP_LEFT_TO_BOTTOM_RIGHT" $font \
            glitter_top_left_to_bottom_right $page(14) $page(16)
    print_page $page(16) "HPDF_TS_REPLACE" $font \
            replace $page(15) ""


    $pdf setPageMode fullscreen

    # save the document to a file
    $pdf saveToFile [file rootname [info script]].pdf

    # clean up
    hpdf::free $pdf

    return 0;
}


main
exit
