# << Haru Free PDF Library 2.0.0 >> -- text_demo.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.

lappend auto_path ..
package require Hpdf

proc show_stripe_pattern {page x y} {
    set iy 0
    while {$iy < 50} {
        $page setRGBStroke 0.0 0.0 0.5
        $page setLineWidth 1
        $page moveTo $x [expr {$y + $iy}]
        $page lineTo \
            [expr {$x + [$page textWidth "ABCabc123"]}] \
            [expr {$y + $iy}]
        $page stroke
        incr iy 3
    }
}

proc show_description {page x y text} {
    set fsize [$page getCurrentFontSize]
    set font [$page getCurrentFont]
    set c [$page getRGBFill]

    $page beginText
    $page setRGBFill 0 0 0
    $page setTextRenderingMode fill
    $page setFontAndSize $font 10
    $page textOut $x [expr {$y - 12}] $text
    $page endText

    $page setFontAndSize $font $fsize
    $page setRGBFill [lindex $c 0] [lindex $c 1] [lindex $c 2]
}

proc main {} {
    set page_title "Text Demo"
    set samp_text  "abcdefgABCDEFG123!#$%&+-@?"
    set samp_text2 "The quick brown fox jumps over the lazy dog."

    set pdf [hpdf::new]
    
    $pdf setCompressionMode all

    set font [$pdf getFont "Helvetica"]

    set page [$pdf addPage]
    
    # NOT IMPREMENTED
    # $pdf print_grid $page

    # print the lines of the page.
    $page setLineWidth 1
    $page rectangle 50 40 \
        [expr {[$page getWidth] -100}]\
        [expr {[$page getHeight] -100}]
    $page stroke

    # print the title of the page (with positioning center).
    $page setFontAndSize $font 24
    set tw [$page textWidth $page_title]
    $page beginText
    $page textOut \
        [expr {([$page getWidth] - $tw)/2}] \
        [expr {[$page getHeight] - 50}] \
        $page_title
    $page endText
    
    $page beginText
    $page moveTextPos 60 [expr {[$page getHeight] - 60}]
    
    #
    # font size
    #
    set fsize 8
    while {$fsize < 60} {
        # set style and size of font.
        $page setFontAndSize $font $fsize

        # set the position of the text.
        $page moveTextPos 0 [expr {-5 - $fsize}]

        # measure the number of characters 
        # which included in the page.
        set len [$page measureText $samp_text \
            [expr {[$page getWidth] - 120}] false]
        incr len -1
        $page showText [string range $samp_text 0 $len]

        # print the description.
        $page moveTextPos 0 -10
        $page setFontAndSize $font 8
        $page showText [format "Fontsize=%.0f" $fsize]

        set fsize [expr {$fsize * 1.5}]
    }
    
    #
    # font color
    #
    $page setFontAndSize $font 8
    $page moveTextPos 0 -30
    $page showText "Font color"

    $page setFontAndSize $font 18
    $page moveTextPos 0 -20
    set len [string length $samp_text]
    for {set i 0} {$i < $len} {incr i} {
        set r [expr {1.0 * $i / $len}]
        set g [expr {1.0 - (1.0 * $i / $len)}]
        $page setRGBFill $r $g 0
        $page showText [string index $samp_text $i]
    }
    $page moveTextPos 0 -25

    for {set i 0} {$i < $len} {incr i} {
        set r [expr {1.0 * $i / $len}]
        set b [expr {1.0 - (1.0 * $i / $len)}]
        $page setRGBFill $r 0 $b
        $page showText [string index $samp_text $i]
    }
    $page moveTextPos 0 -25

    for {set i 0} {$i < $len} {incr i} {
        set b [expr {1.0 * $i / $len}]
        set g [expr {1.0 - (1.0 * $i / $len)}]
        $page setRGBFill 0 $g $b
        $page showText [string index $samp_text $i]
    }
    $page endText

    set ypos 450
    
    #
    # font rendering mode
    #
    $page setFontAndSize $font 32
    $page setRGBFill 0.5 0.5 0.0
    $page setLineWidth 1.5

    # fill
    show_description $page 60 $ypos \
        "RenderingMode=PDF_FILL"
    $page setTextRenderingMode fill
    $page beginText
    $page textOut 60 $ypos "ABCabc123"
    $page endText

    # stroke
    show_description $page 60 [expr {$ypos -50}] \
        "RenderingMode=PDF_STROKE"
    $page setTextRenderingMode stroke
    $page beginText
    $page textOut 60 [expr {$ypos -50}] "ABCabc123"
    $page endText

    # fill_then_stroke
    show_description $page 60 [expr {$ypos -100}] \
        "RenderingMode=PDF_FILL_THEN_STROKE"
    $page setTextRenderingMode fill_then_stroke
    $page beginText
    $page textOut 60 [expr {$ypos -100}] "ABCabc123"
    $page endText

    # fill_clipping
    show_description $page 60 [expr {$ypos -150}] \
        "RenderingMode=PDF_FILL_CLIPPING"
    $page gSave
    $page setTextRenderingMode fill_clipping
    $page beginText
    $page textOut 60 [expr {$ypos -150}] "ABCabc123"
    $page endText
    show_stripe_pattern $page 60 [expr {$ypos - 150}]
    $page gRestore
    
    # stroke_clipping
    show_description $page 60 [expr {$ypos -200}] \
        "RenderingMode=PDF_STROKE_CLIPPING"
    $page gSave
    $page setTextRenderingMode stroke_clipping
    $page beginText
    $page textOut 60 [expr {$ypos -200}] "ABCabc123"
    $page endText
    show_stripe_pattern $page 60 [expr {$ypos - 200}]
    $page gRestore

    # fill_stroke_clipping
    show_description $page 60 [expr {$ypos -250}] \
        "RenderingMode=PDF_FILL_STROKE_CLIPPING"
    $page gSave
    $page setTextRenderingMode fill_stroke_clipping
    $page beginText
    $page textOut 60 [expr {$ypos -250}] "ABCabc123"
    $page endText
    show_stripe_pattern $page 60 [expr {$ypos - 250}]
    $page gRestore
    
    # reset text attributes
    $page setTextRenderingMode fill
    $page setRGBFill 0 0 0
    $page setFontAndSize $font 30

    #
    # rotating text
    #
    set angle1 30 ;#A rotation fo 30 degrees.
    set rad1 [expr {$angle1 / 180.0 * 3.141592}] ;#calcurate the radian value.

    show_description $page 320 [expr {$ypos - 60}] "Rotating text"
    $page beginText
    $page setTextMatrix \
        [expr {cos($rad1)}] \
        [expr {sin($rad1)}] \
        [expr {-sin($rad1)}] \
        [expr {cos($rad1)}] \
        330 \
        [expr {$ypos - 60}]
    $page showText "ABCabc123"
    $page endText
    
    #
    # skewing text
    #
    show_description $page 320 [expr {$ypos - 120}] "Skewing text"
    $page beginText

    set angle1 10
    set angle2 20
    set rad1 [expr {$angle1 / 180.0 * 3.141592}]
    set rad2 [expr {$angle2 / 180.0 * 3.141592}]

    $page setTextMatrix \
        1 \
        [expr {tan($rad1)}] \
        [expr {tan($rad2)}] \
        1 \
        320 \
        [expr {$ypos - 120}]
    $page showText "ABCabc123"
    $page endText

    #
    # scaling text (X direction)
    #
    show_description $page 320 [expr {$ypos - 175}] "Scaling text (X direction)"
    $page beginText
    $page setTextMatrix \
        1.5 \
        0 \
        0 \
        1 \
        320 \
        [expr {$ypos - 175}]
    $page showText "ABCabc12"
    $page endText

    #
    # char spacing, word spacing
    #
    show_description $page 60 140 "char-spacing 0"
    show_description $page 60 100 "char-spacing 1.5"
    show_description $page 60 60 "char-spacing 1.5, word-spacing 2.5"
    
    $page setFontAndSize $font 20
    $page setRGBFill 0.1 0.3 0.1

    # char-spacing 0
    $page beginText
    $page textOut 60 140 $samp_text2
    $page endText

    # char-spacing 1.5
    $page setCharSpace 1.5

    $page beginText
    $page textOut 60 100 $samp_text2
    $page endText

    # char-spacing 1.5, word-spacing 3.5
    $page setWordSpace 2.5

    $page beginText
    $page textOut 60 60 $samp_text2
    $page endText

    $pdf saveToFile [file rootname [info script]].pdf

    hpdf::free $pdf
}

main
