# << Haru Free PDF Library 2.0.0 >> -- text_demo2.tcl
#
# Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
# Copyright (c) 2007      Yoshio Ohtsuka <ohtsuka.yosio@gmail.com>
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation.
# It is provided "as is" without express or implied warranty.

lappend auto_path ..
package require Hpdf

set samp_text "The quick brown fox jumps over the lazy dog. "

set pdf [hpdf::new]

set page [$pdf addPage]
$page setSize a5 portrait
set height [$page getHeight]

set font [$pdf getFont "Helvetica"]
$page setTextLeading 20

# HPDF_TALIGN_LEFT
set left  25
set top   545
set right 200
set bottom [expr {$top - 40}]

$page rectangle \
    $left \
    $bottom \
    [expr {$right - $left}] \
    [expr {$top - $bottom}]
$page stroke

$page beginText
$page setFontAndSize $font 10
$page textOut $left [expr {$top + 3}] "HPDF_TALIGN_LEFT"

$page setFontAndSize $font 13
$page textRect $left $top $right $bottom $samp_text left
$page endText

# HPDF_TALIGN_RIGHT
set left  220
set right 395

$page rectangle \
    $left \
    $bottom \
    [expr {$right - $left}] \
    [expr {$top - $bottom}]
$page stroke

$page beginText
$page setFontAndSize $font 10
$page textOut $left [expr {$top + 3}] "HPDF_TALIGN_RIGHT"

$page setFontAndSize $font 13
$page textRect $left $top $right $bottom $samp_text right
$page endText

# HPDF_TALIGN_CENTER
set left   25
set top    475
set right  200
set bottom [expr {$top - 40}]

$page rectangle \
    $left \
    $bottom \
    [expr {$right - $left}] \
    [expr {$top - $bottom}]
$page stroke

$page beginText
$page setFontAndSize $font 10
$page textOut $left [expr {$top + 3}] "HPDF_TALIGN_CENTER"

$page setFontAndSize $font 13
$page textRect \
    $left \
    $top \
    $right \
    $bottom \
    $samp_text \
    center
$page endText

# HPDF_TALIGN_JUSTIFY
set left  220
set right 395

$page rectangle \
    $left \
    $bottom \
    [expr {$right - $left}] \
    [expr {$top - $bottom}]
$page stroke

$page beginText
$page setFontAndSize $font 10
$page textOut $left [expr {$top + 3}] "HPDF_TALIGN_JUSTIFY"

$page setFontAndSize $font 13
$page textRect \
    $left \
    $top \
    $right \
    $bottom \
    $samp_text \
    justify
$page endText


# Skewed coordinate system

set angle1 5.0
set angle2 10.0
set rad1 [expr {$angle1 / 180.0 * 3.141592}]
set rad2 [expr {$angle2 / 180.0 * 3.141592}]

$page gSave

$page concat 1 [expr {tan($rad1)}] [expr {tan($rad2)}] 1 25 350
set left   0
set top    40
set right  175
set bottom 0

$page rectangle \
    $left \
    $bottom \
    [expr {$right - $left}] \
    [expr {$top - $bottom}]
$page stroke

$page beginText

$page setFontAndSize $font 10
$page textOut $left [expr {$top + 3}] "Skewed coordinate system"

$page setFontAndSize $font 13
$page textRect $left $top $right $bottom $samp_text left

$page endText

$page gRestore


# Rotated coordinate system
$page gSave

set angle1 5.0
set rad1 [expr {$angle1 / 180.0 * 3.141592}]

$page concat \
    [expr {cos($rad1)}] \
    [expr {sin($rad1)}] \
    [expr {-sin($rad1)}] \
    [expr {cos($rad1)}] \
    220 \
    350
set left   0
set top    40
set right  175
set bottom 0

$page rectangle \
    $left \
    $bottom \
    [expr {$right - $left}] \
    [expr {$top - $bottom}]
$page stroke

$page beginText

$page setFontAndSize $font 10
$page textOut $left [expr {$top + 3}] "Rotated coordinate system"

$page setFontAndSize $font 13
$page textRect $left $top $right $bottom $samp_text left

$page endText

$page gRestore


# text along a circle
$page setGrayStroke 0
$page circle 210 190 145
$page circle 210 190 113
$page stroke

set angle1 [expr {360.0 / [string length $samp_text]}]
set angle2 180.0

$page beginText
set font [$pdf getFont "Courier-Bold"]
$page setFontAndSize $font 30

set i 0
set len [string length $samp_text]
while {$i < $len} {
    set rad1 [expr {($angle2 - 90) / 180 * 3.141592}]
    set rad2 [expr {$angle2 / 180 * 3.141592}]

    set x [expr {210.0 + cos($rad2) * 122}]
    set y [expr {190.0 + sin($rad2) * 122}]

    $page setTextMatrix \
        [expr {cos($rad1)}]\
        [expr {sin($rad1)}]\
        [expr {-sin($rad1)}]\
        [expr {cos($rad1)}]\
        $x\
        $y

    $page showText [string index $samp_text $i]
    set angle2 [expr {$angle2 - $angle1}]
    incr i
}

$page endText

$pdf saveToFile [file rootname [info script]].pdf
