#include "tclhpdf.h"

static int hpdf_GetVersionObjCmd(ClientData clientData, 
        Tcl_Interp *interp, int objc, Tcl_Obj *const objv[]);
static int hpdf_NewDocObjCmd(ClientData clientData, 
        Tcl_Interp *interp, int objc, Tcl_Obj *const objv[]);
static int hpdf_FreeObjCmd(ClientData clientData, 
        Tcl_Interp *interp, int objc, Tcl_Obj *const objv[]);


static void
hpdf_error_handler  (HPDF_STATUS  error_no,
                     HPDF_STATUS  detail_no,
                     void         *user_data)
{
/*    printf ("errorno=%04X, detail_no=%04X\n", error_no, detail_no);*/
    Tcl_Interp *interp = (Tcl_Interp*)user_data;
    Tcl_AppendResult(interp, hpdf_getErrorMessage(error_no), (char*)NULL);
}


static int
hpdf_GetVersionObjCmd(ClientData clientData, Tcl_Interp *interp, 
        int objc, Tcl_Obj *const objv[])
{
    if (objc != 1) {
        Tcl_WrongNumArgs(interp, 1, objv, NULL);
        return TCL_ERROR;
    }
    Tcl_SetObjResult(interp, Tcl_NewStringObj(HPDF_GetVersion(), -1));
    return TCL_OK;
}

static int
hpdf_NewDocObjCmd(ClientData clientData, Tcl_Interp *interp,
        int objc, Tcl_Obj *const objv[])
{
    HPDF_Doc pdf;
    if (objc != 1) {
        Tcl_WrongNumArgs(interp, 1, objv, NULL);
        return TCL_ERROR;
    }

    pdf = HPDF_New(hpdf_error_handler, interp);

    if (!pdf) {
        return TCL_ERROR;
    }
    
    if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_DOC, NULL, pdf) != TCL_OK) {
        return TCL_ERROR;
    }
    return TCL_OK;
}

static int
hpdf_FreeObjCmd(ClientData clientData, Tcl_Interp *interp,
        int objc, Tcl_Obj *const objv[])
{
    hpdf_Obj *hpdfObj;
    char *cmdname;

    if (objc != 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "hpdfDocObj");
        return TCL_ERROR;
    }
    
    /* Validate command */
    if (hpdf_GetObjFromCmdName (interp, objv[1], 
                TCLHPDF_TYPE_DOC, &hpdfObj) != TCL_OK) {
        return TCL_ERROR;
    }

    cmdname = Tcl_GetString(objv[1]);
    if (Tcl_DeleteCommand(interp, cmdname) != TCL_OK) {
        Tcl_AppendResult(interp, "failed to delete cmd: ",
                cmdname, NULL);
        return TCL_ERROR;
    }
    return TCL_OK;
}


/*
 *----------------------------------------------------------------------
 */
DLLEXPORT int
Tclhpdf_Init (Tcl_Interp *interp)
{
#ifdef USE_TCL_STUBS
    if (Tcl_InitStubs(interp, "8.1", 0) == NULL) {
        return TCL_ERROR;
    }
#endif

    if (Tcl_PkgRequire(interp, "Tcl", "8.1", 0) == NULL) {
        return TCL_ERROR;
    }

    if (Tcl_PkgProvide(interp, "Hpdf", PACKAGE_VERSION) 
            != TCL_OK) {
        return TCL_ERROR;
    }

    if (Tcl_Eval(interp, "namespace eval ::hpdf {}") 
            != TCL_OK) {
        return TCL_ERROR;
    }

    Tcl_CreateObjCommand(interp, "::hpdf::getVersion", 
        (Tcl_ObjCmdProc *) hpdf_GetVersionObjCmd,
        (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

    Tcl_CreateObjCommand(interp, "::hpdf::new", 
        (Tcl_ObjCmdProc *) hpdf_NewDocObjCmd,
        (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

    Tcl_CreateObjCommand(interp, "::hpdf::free", 
        (Tcl_ObjCmdProc *) hpdf_FreeObjCmd,
        (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

    return TCL_OK;
}

