#include "tclhpdf.h"

int hpdf_AnnotationObjCmd(ClientData clientData, Tcl_Interp *interp, 
        int objc, Tcl_Obj *const objv[])
{
    int index;
    HPDF_STATUS ret;
    HPDF_Annotation annot = TCLHPDFOBJ_GET_ANNOTATION(clientData);
    HPDF_Doc pdf = TCLHPDFOBJ_GET_ROOTDOC(clientData);

    static CONST char *optionStrings[] = {
    "setHighlightMode",
    "setBorderStyle",
    "setIcon",
    "setOpened",
     (char*)NULL
    };

    enum options {
    TCLHPDF_LINKANNOT_SETHIGHLIGHTMODE,
    TCLHPDF_LINKANNOT_SETBORDERSTYLE,
    TCLHPDF_TEXTANNOT_SETICON,
    TCLHPDF_TEXTANNOT_SETOPENED
    };

    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
        return TCL_ERROR;
    }
    if (Tcl_GetIndexFromObj(interp, objv[1], optionStrings, 
                "option", 0, &index) != TCL_OK) {
        return TCL_ERROR;
    }

    switch (index) {
        case TCLHPDF_LINKANNOT_SETHIGHLIGHTMODE:
        {
            int mode;
            TCLHPDF_ARGCHECK(3,3,2,"mode")

            if (hpdf_getEnumFromStringObj (interp, 
                optionAnnotHighlightModeString, (int*)optionAnnotHighlightModeEnum,
                objv[2], &mode) != TCL_OK) {
                return TCL_ERROR;
            }

            ret = HPDF_LinkAnnot_SetHighlightMode(annot, mode);
            hpdf_ErrorReturn2(interp, pdf, ret);
            return TCL_OK;
        }

        case TCLHPDF_LINKANNOT_SETBORDERSTYLE:
        {
            double width;
            int dashon, dashoff;
            TCLHPDF_ARGCHECK(5,5,2,"width dash_on dash_off")
            
            if (Tcl_GetDoubleFromObj(interp, objv[2], &width) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[3], &dashon) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[4], &dashoff) != TCL_OK) {
                return TCL_ERROR;
            }

            ret = HPDF_LinkAnnot_SetBorderStyle(annot, width, dashon, dashoff);
            hpdf_ErrorReturn2(interp, pdf, ret);
            return TCL_OK;
        }

        case TCLHPDF_TEXTANNOT_SETICON:
        {
            int icon;
            TCLHPDF_ARGCHECK(3,3,2,"icon")

            if (hpdf_getEnumFromStringObj (interp, 
                optionAnnotIconString, (int*)optionAnnotIconEnum,
                objv[2], &icon) != TCL_OK) {
                return TCL_ERROR;
            }

            ret = HPDF_TextAnnot_SetIcon(annot, icon);
            hpdf_ErrorReturn2(interp, pdf, ret);
            return TCL_OK;
        }

        case TCLHPDF_TEXTANNOT_SETOPENED:
        {
            int opened;
            TCLHPDF_ARGCHECK(3,3,2,"opened")

            if (Tcl_GetBooleanFromObj(interp, objv[2], &opened) != TCL_OK) {
                return TCL_ERROR;
            }
            ret = HPDF_TextAnnot_SetOpened(annot, opened);
            hpdf_ErrorReturn2(interp, pdf, ret);
            return TCL_OK;
        }

        default: break;
    }
    return TCL_OK;
}
