#include "tclhpdf.h"

int hpdf_DestinationObjCmd(ClientData clientData, Tcl_Interp *interp, 
        int objc, Tcl_Obj *const objv[])
{
    int index;
    HPDF_Destination dst = TCLHPDFOBJ_GET_DESTINATION(clientData);
    HPDF_Doc pdf = TCLHPDFOBJ_GET_ROOTDOC(clientData);
    HPDF_STATUS ret;

    static CONST char *optionStrings[] = {
    "setXYZ",
    "setFit",
    "setFitH",
    "setFitV",
    "setFitR",
    "setFitB",
    "setFitBH",
    "setFitBV",
     (char*)NULL
    };

    enum options {
    TCLHPDF_DESTINATION_SETXYZ,
    TCLHPDF_DESTINATION_SETFIT,
    TCLHPDF_DESTINATION_SETFITH,
    TCLHPDF_DESTINATION_SETFITV,
    TCLHPDF_DESTINATION_SETFITR,
    TCLHPDF_DESTINATION_SETFITB,
    TCLHPDF_DESTINATION_SETFITBH,
    TCLHPDF_DESTINATION_SETFITBV
    };

    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
        return TCL_ERROR;
    }
    if (Tcl_GetIndexFromObj(interp, objv[1], optionStrings, 
                "option", 0, &index) != TCL_OK) {
        return TCL_ERROR;
    }

    switch (index) {
        case TCLHPDF_DESTINATION_SETXYZ:
        {
            double left, top, zoom;
            TCLHPDF_ARGCHECK(5,5,2,"left top zoom")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &left) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &top) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &zoom) != TCL_OK) {
                return TCL_ERROR;
            }
            if (zoom < 0.08 || 32.0 < zoom) {
                Tcl_AppendResult(interp, "zoom must be between 0.08 and 32.0.", NULL);
                return TCL_ERROR;
            }

            ret = HPDF_Destination_SetXYZ(dst, left, top, zoom);
            hpdf_ErrorReturn2(interp, pdf,ret);
            return TCL_OK;
        }

        case TCLHPDF_DESTINATION_SETFIT:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            ret = HPDF_Destination_SetFit(dst);
            hpdf_ErrorReturn2(interp, pdf, ret);
            return TCL_OK;

        case TCLHPDF_DESTINATION_SETFITH:
        {
            double top;
            TCLHPDF_ARGCHECK(3,3,2,"top")
            
            if (Tcl_GetDoubleFromObj(interp, objv[2], &top) != TCL_OK) {
                return TCL_ERROR;
            }
            ret = HPDF_Destination_SetFitH(dst, top);
            hpdf_ErrorReturn2(interp, pdf, ret);
            return TCL_OK;
        }

        case TCLHPDF_DESTINATION_SETFITV:
        {
            double left;
            TCLHPDF_ARGCHECK(3,3,2,"left")
            
            if (Tcl_GetDoubleFromObj(interp, objv[2], &left) != TCL_OK) {
                return TCL_ERROR;
            }
            ret = HPDF_Destination_SetFitV(dst, left);
            hpdf_ErrorReturn2(interp, pdf, ret);
            return TCL_OK;
        }

        case TCLHPDF_DESTINATION_SETFITR:
        {
            double left, bottom, right, top;
            TCLHPDF_ARGCHECK(6,6,2,"left bottom right top")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &left) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &bottom) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &right) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &top) != TCL_OK) {
                return TCL_ERROR;
            }
            ret = HPDF_Destination_SetFitR(dst, left, bottom, right, top);
            hpdf_ErrorReturn2(interp, pdf, ret);
            return TCL_OK;
        }

        case TCLHPDF_DESTINATION_SETFITB:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            ret = HPDF_Destination_SetFitB(dst);
            hpdf_ErrorReturn2(interp, pdf, ret);
            return TCL_OK;

        case TCLHPDF_DESTINATION_SETFITBH:
        {
            double top;
            TCLHPDF_ARGCHECK(3,3,2,"top")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &top) != TCL_OK) {
                return TCL_ERROR;
            }
            ret = HPDF_Destination_SetFitBH(dst, top);
            hpdf_ErrorReturn2(interp, pdf, ret);
            return TCL_OK;
        }

        case TCLHPDF_DESTINATION_SETFITBV:
        {
            double left;
            TCLHPDF_ARGCHECK(3,3,2,"left")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &left) != TCL_OK) {
                return TCL_ERROR;
            }
            ret = HPDF_Destination_SetFitBH(dst, left);
            hpdf_ErrorReturn2(interp, pdf, ret);
            return TCL_OK;
        }
        default: break;
    }
    return TCL_OK;
}
