#include "tclhpdf.h"


void 
hpdf_DocCmdDeleteProc(ClientData clientData)
{
    if (clientData) 
        hpdf_ObjDestroy((hpdf_Obj*)clientData, 0);
}

int
hpdf_DocObjCmd(ClientData clientData, Tcl_Interp *interp,
        int objc, Tcl_Obj *const objv[])
{
    int index;
    HPDF_Doc pdf = TCLHPDFOBJ_GET_DOC(clientData);
    HPDF_STATUS ret;

    static CONST char *optionStrings[] = {
    "newDoc", "freeDoc", "hasDoc", "freeDocAll",
    "saveToStream", "getStreamSize", "readFromStream", "resetStream", "saveToFile",
/*    "setErrorHandler", "getError", "getErrorDetail", "resetError",*/
    "setPagesConfiguration",
    "getPageLayout", "setPageLayout",
    "getPageMode", "setPageMode",
    "getViewerPreference", "setViewerPreference",
    "setOpenAction",
    "getPageByIndex", "getCurrentPage", "addPage", "insertPage",
    "getFont", "loadType1FontFromFile",
    "loadTTFontFromFile", "loadTTFontFromFile2",
    "addPageLabel",
    "useJPFonts", "useKRFonts", "useCNSFonts", "useCNTFonts",
    "createOutline",
    "getEncoder", "getCurrentEncoder", "setCurrentEncoder",
    "useJPEncodings", "useKREncodings", "useCNSEncodings", "useCNTEncodings",
    "loadPngImageFromFile", "loadPngImageFromFile2", "loadJpegImageFromFile",
    "loadRawImageFromFile", "loadRawImageFromMem",
    "setInfoAttr", "getInfoAttr", "setInfoDateAttr",
    "setPassword", "setPermission", "setEncryptionMode",
    "setCompressionMode", "createExtGState", (char*)NULL
    };
    enum options {
    TCLHPDF_DOC_NEW_DOC,
    TCLHPDF_DOC_FREE_DOC,
    TCLHPDF_DOC_HAS_DOC,
    TCLHPDF_DOC_FREE_DOC_ALL,
    TCLHPDF_DOC_SAVE_TO_STREAM,
    TCLHPDF_DOC_GET_STREAM_SIZE,
    TCLHPDF_DOC_READ_FROM_STREAM,
    TCLHPDF_DOC_RESET_STREAM,
    TCLHPDF_DOC_SAVE_TO_FILE,
/*    TCLHPDF_DOC_SET_ERROR_HANDLER,*/
/*    TCLHPDF_DOC_GET_ERROR,*/
/*    TCLHPDF_DOC_GET_ERROR_DETAIL,*/
/*    TCLHPDF_DOC_RESET_ERROR,*/
    TCLHPDF_DOC_SET_PAGES_CONFIGURATION,
    TCLHPDF_DOC_GET_PAGE_LAYOUT,
    TCLHPDF_DOC_SET_PAGE_LAYOUT,
    TCLHPDF_DOC_GET_PAGE_MODE,
    TCLHPDF_DOC_SET_PAGE_MODE,
    TCLHPDF_DOC_GET_VIEWER_PREFERENCE,
    TCLHPDF_DOC_SET_VIEWER_PREFERENCE,
    TCLHPDF_DOC_SET_OPEN_ACTION,
    TCLHPDF_DOC_GET_PAGE_BY_INDEX,
    TCLHPDF_DOC_GET_CURRENT_PAGE,
    TCLHPDF_DOC_ADD_PAGE,
    TCLHPDF_DOC_INSERT_PAGE,
    TCLHPDF_DOC_GET_FONT,
    TCLHPDF_DOC_LOAD_TYPE1FONT_FROM_FILE,
    TCLHPDF_DOC_LOAD_TTFONT_FROM_FILE,
    TCLHPDF_DOC_LOAD_TTFONT_FROM_FILE2,
    TCLHPDF_DOC_ADD_PAGE_LABEL,
    TCLHPDF_DOC_USE_JP_FONTS,
    TCLHPDF_DOC_USE_KR_FONTS,
    TCLHPDF_DOC_USE_CNS_FONTS,
    TCLHPDF_DOC_USE_CNT_FONTS,
    TCLHPDF_DOC_CREATE_OUTLINE,
    TCLHPDF_DOC_GET_ENCODER,
    TCLHPDF_DOC_GET_CURRENT_ENCODER,
    TCLHPDF_DOC_SET_CURRENT_ENCODER,
    TCLHPDF_DOC_USE_JP_ENCODINGS,
    TCLHPDF_DOC_USE_KR_ENCODINGS,
    TCLHPDF_DOC_USE_CNS_ENCODINGS,
    TCLHPDF_DOC_USE_CNT_ENCODINGS,
    TCLHPDF_DOC_LOAD_PNG_IMAGE_FROM_FILE,
    TCLHPDF_DOC_LOAD_PNG_IMAGE_FROM_FILE2,
    TCLHPDF_DOC_LOAD_JPEG_IMAGE_FROM_FILE,
    TCLHPDF_DOC_LOAD_RAW_IMAGE_FROM_FILE,
    TCLHPDF_DOC_LOAD_RAW_IMAGE_FROM_MEM,
    TCLHPDF_DOC_SET_INFO_ATTR,
    TCLHPDF_DOC_GET_INFO_ATTR,
    TCLHPDF_DOC_SET_INFO_DATE_ATTR,
    TCLHPDF_DOC_SET_PASSWORD,
    TCLHPDF_DOC_SET_PERMISSION,
    TCLHPDF_DOC_SET_ENCRYPTION_MODE,
    TCLHPDF_DOC_SET_COMPRESSION_MODE,
    TCLHPDF_DOC_CREATE_EXT_GSTATE
    };

    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
        return TCL_ERROR;
    }
    if (Tcl_GetIndexFromObj(interp, objv[1], optionStrings, 
                "option", 0, &index) != TCL_OK) {
        return TCL_ERROR;
    }

    switch (index) {
        case TCLHPDF_DOC_NEW_DOC:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            HPDF_NewDoc(pdf);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_DOC_FREE_DOC:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            HPDF_FreeDoc(pdf);
            return TCL_OK;

        case TCLHPDF_DOC_HAS_DOC:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            Tcl_SetObjResult(interp, 
                    Tcl_NewIntObj(HPDF_HasDoc(pdf)));
            return TCL_OK;


        case TCLHPDF_DOC_FREE_DOC_ALL:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            HPDF_FreeDocAll(pdf);
            return TCL_OK;

        case TCLHPDF_DOC_SAVE_TO_STREAM:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            HPDF_SaveToStream(pdf);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_DOC_GET_STREAM_SIZE:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            Tcl_SetObjResult(interp, 
                    Tcl_NewIntObj(HPDF_GetStreamSize(pdf)));
            return TCL_OK;

        case TCLHPDF_DOC_READ_FROM_STREAM:
        {
            HPDF_UINT32 size;
            HPDF_BYTE *buf = NULL;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            size = HPDF_GetStreamSize(pdf);
            if (size == 0) {
                Tcl_SetObjResult(interp, Tcl_NewObj());
                return TCL_OK;
            }
            buf = malloc(size);
            ret = HPDF_ReadFromStream(pdf, buf, &size);
            if (ret == HPDF_OK) {
                Tcl_SetObjResult(interp, 
                        Tcl_NewByteArrayObj(buf, size));
            }
            if (buf) free(buf);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_RESET_STREAM:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            HPDF_ResetStream(pdf);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_DOC_SAVE_TO_FILE:
        {
            char *filename;
            Tcl_DString ds;
            TCLHPDF_ARGCHECK(3,3,2,"filename")

            filename = Tcl_UtfToExternalDString(NULL, 
                    Tcl_GetString(objv[2]), -1, &ds);
            HPDF_SaveToFile(pdf, filename);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

/*        case TCLHPDF_DOC_SET_ERROR_HANDLER:*/
/*        case TCLHPDF_DOC_GET_ERROR:*/
/*        case TCLHPDF_DOC_GET_ERROR_DETAIL:*/
/*        case TCLHPDF_DOC_RESET_ERROR:*/

        case TCLHPDF_DOC_SET_PAGES_CONFIGURATION:
        {
            int n;
            TCLHPDF_ARGCHECK(3,3,2,"page_par_pages")

            if (Tcl_GetIntFromObj(interp, objv[2], &n) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_SetPagesConfiguration(pdf, n);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_GET_PAGE_LAYOUT:
        {
            HPDF_UINT layout;
            Tcl_Obj *layoutstring;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            layout = HPDF_GetPageLayout(pdf);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_getStringObjFromEnum (interp,
                optionPageLayoutString, (int*)optionPageLayoutEnum, 
                layout, &layoutstring) != TCL_OK) {
                Tcl_AppendResult(interp, "page layout isn't set or unknown.", NULL);
                return TCL_ERROR;
            }
            Tcl_SetObjResult(interp, layoutstring);
            return TCL_OK;
        }

        case TCLHPDF_DOC_SET_PAGE_LAYOUT:
        {
            HPDF_UINT layout;
            TCLHPDF_ARGCHECK(3,3,2,"layout")

            if (hpdf_getEnumFromStringObj (interp, 
                optionPageLayoutString, (int*)optionPageLayoutEnum,
                objv[2], &layout) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_SetPageLayout(pdf, layout);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_GET_PAGE_MODE:
        {
            HPDF_UINT mode;
            Tcl_Obj *modestring;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            mode = HPDF_GetPageMode(pdf);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_getStringObjFromEnum (interp,
                optionPageModeString, (int*)optionPageModeEnum, 
                mode, &modestring) != TCL_OK) {
                Tcl_AppendResult(interp, "page mode isn't set or unknown.", NULL);
                return TCL_ERROR;
            }
            Tcl_SetObjResult(interp, modestring);
            return TCL_OK;
        }

        case TCLHPDF_DOC_SET_PAGE_MODE:
        {
            HPDF_UINT mode;
            TCLHPDF_ARGCHECK(3,3,2,"mode")

            if (hpdf_getEnumFromStringObj (interp, 
                optionPageModeString, (int*)optionPageModeEnum,
                objv[2], &mode) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_SetPageMode(pdf, mode);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_GET_VIEWER_PREFERENCE:
        {
            HPDF_UINT mode;
            Tcl_Obj *modeobj;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            mode = HPDF_GetViewerPreference(pdf);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_getListObjFromFlags (interp,
                optionViewerPreferenceString, (int*)optionViewerPreferenceEnum, 
                mode, &modeobj) != TCL_OK) {
                return TCL_ERROR;
            }
            Tcl_SetObjResult(interp, modeobj);
            return TCL_OK;
        }

        case TCLHPDF_DOC_SET_VIEWER_PREFERENCE:
        {
            HPDF_UINT mode;
            TCLHPDF_ARGCHECK(3,3,2,"flags")

            if (hpdf_getFlagsFromListObj (interp,
                optionViewerPreferenceString, (int*)optionViewerPreferenceEnum, 
                objv[2], &mode) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_SetViewerPreference(pdf, mode);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_SET_OPEN_ACTION:
        {
            hpdf_Obj *obj;
            TCLHPDF_ARGCHECK(3,3,2,"hpdfDestinationObj")

            if (hpdf_GetObjFromCmdName (interp, objv[2], 
                        TCLHPDF_TYPE_DESTINATION, &obj) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_SetOpenAction(pdf, TCLHPDFOBJ_GET_DESTINATION(obj));
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_GET_PAGE_BY_INDEX:
        {
            int i;
            HPDF_Page page;
            TCLHPDF_ARGCHECK(3,3,2,"index")

            if (Tcl_GetIntFromObj(interp, objv[2], &i) != TCL_OK) {
                return TCL_ERROR;
            }
            page = HPDF_GetPageByIndex(pdf, i);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_PAGE,
                        clientData, page)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_DOC_GET_CURRENT_PAGE:
        {
            HPDF_Page page;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            page = HPDF_GetCurrentPage(pdf);
            hpdf_ErrorReturn(interp, pdf);
            /* FIXME:If there is no page, error callback function
             * isn't fired. and returns NULL. But HPDF_GetPageByIndex
             * fire the error callbacks, if there is no page. 
             * We should fire error.
             */
            if (!page) {
                Tcl_SetObjResult(interp, 
                        Tcl_NewStringObj("hpdf doc doesnt have page",-1));
                return TCL_ERROR;
            }

            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_PAGE,
                        clientData, page)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_DOC_ADD_PAGE:
        {
            HPDF_Page page;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            page = HPDF_AddPage(pdf);
            hpdf_ErrorReturn(interp, pdf);
            /* FIXME:If there is no page, error callback function
             * isn't fired. and returns NULL. But HPDF_GetPageByIndex
             * fire the error callbacks, if there is no page. 
             * We should fire error.
             */
            if (!page) {
                Tcl_SetObjResult(interp, 
                        Tcl_NewStringObj("hpdf doc doesnt have page",-1));
                return TCL_ERROR;
            }
            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_PAGE,
                        clientData, page)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_DOC_INSERT_PAGE:
        {
            hpdf_Obj *obj;
            HPDF_Page page;
            TCLHPDF_ARGCHECK(3,3,2,"hpdfPageObj")

            if (hpdf_GetObjFromCmdName (interp, objv[2], 
                TCLHPDF_TYPE_PAGE, &obj) != TCL_OK) {
                return TCL_ERROR;
            }
            page = HPDF_InsertPage(pdf, TCLHPDFOBJ_GET_PAGE(obj));
            hpdf_ErrorReturn(interp, pdf);
            /* FIXME:If there is no page, error callback function
             * isn't fired. and returns NULL. But HPDF_GetPageByIndex
             * fire the error callbacks, if there is no page. 
             * We should fire error.
             */
            if (!page) {
                Tcl_SetObjResult(interp, 
                        Tcl_NewStringObj("hpdf doc doesnt have page",-1));
                return TCL_ERROR;
            }

            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_PAGE,
                        clientData, page)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_DOC_GET_FONT:
        {
            HPDF_Font font;
            const char *fontname, *encodingname;
            Tcl_DString ds1, ds2;
            TCLHPDF_ARGCHECK(3,4,2,"fontname ?encodingname?")

            fontname = Tcl_UtfToExternalDString(NULL, 
                    Tcl_GetString(objv[2]), -1, &ds1);

            if (objc == 4) {
                encodingname = Tcl_UtfToExternalDString(NULL, 
                    Tcl_GetString(objv[3]), -1, &ds2);
            }
            
            if (objc == 4 && strlen(encodingname) > 0) {
                font = HPDF_GetFont(pdf, fontname, encodingname);
            } else {
                font = HPDF_GetFont(pdf, fontname, NULL);
            }
            Tcl_DStringFree(&ds1);
            if (objc == 4) {
                Tcl_DStringFree(&ds2);
            }
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_FONT,
                        clientData, font)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_DOC_LOAD_TYPE1FONT_FROM_FILE:
        {
            const char *fontname;
            const char *afm, *pfm;
            Tcl_DString ds1, ds2;
            Tcl_Obj *result;
            TCLHPDF_ARGCHECK(4,4,2,"afmfilename pfmfilename")

            afm = Tcl_UtfToExternalDString(NULL, Tcl_GetString(objv[2]), -1, &ds1);
            pfm = Tcl_UtfToExternalDString(NULL, Tcl_GetString(objv[3]), -1, &ds2);

            fontname = HPDF_LoadType1FontFromFile(pdf, afm, pfm);
            result = Tcl_NewStringObj(
                    Tcl_ExternalToUtfDString(NULL, fontname, -1, &ds1), -1);
            Tcl_DStringFree(&ds1);
            Tcl_DStringFree(&ds2);
            hpdf_ErrorReturn(interp, pdf);

            Tcl_SetObjResult(interp, result);
            return TCL_OK;
        }

        case TCLHPDF_DOC_LOAD_TTFONT_FROM_FILE:
        {
            const char *fontname;
            const char *filename;
            Tcl_DString ds;
            Tcl_Obj *result;
            int embed;
            TCLHPDF_ARGCHECK(4,4,2,"filename embedding")

            filename = Tcl_UtfToExternalDString(NULL, 
                    Tcl_GetString(objv[2]), -1, &ds);
            if (Tcl_GetBooleanFromObj(interp, objv[3], &embed) != TCL_OK) {
                return TCL_ERROR;
            }
            fontname = HPDF_LoadTTFontFromFile(pdf, filename, embed);
            result = Tcl_NewStringObj(
                    Tcl_ExternalToUtfDString(NULL, fontname, -1, &ds), -1);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            Tcl_SetObjResult(interp, result);
            return TCL_OK;
        }

        case TCLHPDF_DOC_LOAD_TTFONT_FROM_FILE2:
        {
            const char *fontname;
            const char *filename;
            Tcl_DString ds;
            Tcl_Obj *result;
            int index, embed;
            TCLHPDF_ARGCHECK(5,5,2,"filename index embedding")

            filename = Tcl_UtfToExternalDString(NULL, 
                    Tcl_GetString(objv[2]), -1, &ds);
            if (Tcl_GetIntFromObj(interp, objv[3], &index) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetBooleanFromObj(interp, objv[4], &embed) != TCL_OK) {
                return TCL_ERROR;
            }
            fontname = HPDF_LoadTTFontFromFile2(pdf, filename, index, embed);
            result = Tcl_NewStringObj(
                    Tcl_ExternalToUtfDString(NULL, fontname, -1, &ds), -1);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            Tcl_SetObjResult(interp, result);
            return TCL_OK;
        }

        case TCLHPDF_DOC_ADD_PAGE_LABEL:
        {
            Tcl_DString ds;
            int num, style, first;
            const char *prefix;
            TCLHPDF_ARGCHECK(6,6,2,"pagenum style firstpage prefix")
            
            if (Tcl_GetIntFromObj(interp, objv[2], &num) != TCL_OK) {
                return TCL_ERROR;
            }
            if (hpdf_getEnumFromStringObj (interp, 
                optionPageNumStyleString, (int*)optionPageNumStyleEnum,
                objv[3], &style) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[4], &first) != TCL_OK) {
                return TCL_OK;
            }
            prefix = Tcl_UtfToExternalDString(NULL, 
                    Tcl_GetString(objv[5]), -1, &ds);

            HPDF_AddPageLabel(pdf, num, style, first, prefix);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_USE_JP_FONTS:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            HPDF_UseJPFonts(pdf);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_DOC_USE_KR_FONTS:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
             
            HPDF_UseKRFonts(pdf);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_DOC_USE_CNS_FONTS:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            HPDF_UseCNSFonts(pdf);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_DOC_USE_CNT_FONTS:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            HPDF_UseCNTFonts(pdf);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_DOC_CREATE_OUTLINE:
        {
            HPDF_Outline outline, parent;
            HPDF_Encoder encoder;
            hpdf_Obj *encoderobj, *parentobj;
            const char *title;
            Tcl_DString ds;
            TCLHPDF_ARGCHECK(4,5,2,"parent title ?encoder?")

            if (strlen(Tcl_GetString(objv[2])) > 0) {
                if (hpdf_GetObjFromCmdName (interp, objv[2], 
                    TCLHPDF_TYPE_OUTLINE, &parentobj) != TCL_OK) {
                    return TCL_ERROR;
                }
                parent = TCLHPDFOBJ_GET_OUTLINE(parentobj);
            } else {
                parent = NULL;
            }

            title = hpdf_TclObjToHpdfText(objv[3], &ds);
            
            if (objc == 5 && strlen(Tcl_GetString(objv[4])) > 0) {
                if (hpdf_GetObjFromCmdName (interp, objv[4], 
                    TCLHPDF_TYPE_ENCODER, &encoderobj) != TCL_OK) {
                    Tcl_DStringFree(&ds);
                    return TCL_ERROR;
                }
                encoder = TCLHPDFOBJ_GET_ENCODER(encoderobj);
            } else {
                encoder = NULL;
            }
            outline = HPDF_CreateOutline(pdf, parent, title, encoder);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            if (!outline) {
                Tcl_SetObjResult(interp, 
                        Tcl_NewStringObj("hpdf doc failed to create outline",-1));
                return TCL_ERROR;
            }
            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_OUTLINE,
                        clientData, outline)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_DOC_GET_ENCODER:
        {
            Tcl_DString ds;
            HPDF_Encoder encoder;
            const char *encoding_name;
            TCLHPDF_ARGCHECK(3,3,2,"encodingname")

            encoding_name = Tcl_UtfToExternalDString(NULL, 
                    Tcl_GetString(objv[2]), -1, &ds);

            encoder = HPDF_GetEncoder(pdf, encoding_name);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            if (!encoder) {
                Tcl_SetObjResult(interp, 
                        Tcl_NewStringObj("hpdf doc failed to create encoder",-1));
                return TCL_ERROR;
            }

            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_ENCODER,
                        clientData, encoder)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }
        
        case TCLHPDF_DOC_GET_CURRENT_ENCODER:
        {
            HPDF_Encoder encoder;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            encoder = HPDF_GetCurrentEncoder(pdf);
            hpdf_ErrorReturn(interp, pdf);

            if (!encoder) {
                Tcl_SetObjResult(interp, 
                        Tcl_NewStringObj("hpdf doc failed to create encoder",-1));
                return TCL_ERROR;
            }

            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_ENCODER,
                        clientData, encoder)!=TCL_OK) {
                return TCL_ERROR;
            }

            return TCL_OK;
        }

        case TCLHPDF_DOC_SET_CURRENT_ENCODER:
        {
            Tcl_DString ds;
            const char *encoding_name;
            TCLHPDF_ARGCHECK(3,3,2,"encodingname")

            encoding_name = Tcl_UtfToExternalDString(NULL, 
                    Tcl_GetString(objv[2]), -1, &ds);

            /* FIXME: hpdf manual said that this function
             * returns outline handler. but it returns 
             * HPDF_STATUS. The document may be wrong.
             */

            HPDF_SetCurrentEncoder(pdf, encoding_name);
            Tcl_DStringFree(&ds);

            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_USE_JP_ENCODINGS:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            HPDF_UseJPEncodings(pdf);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_DOC_USE_KR_ENCODINGS:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            HPDF_UseKREncodings(pdf);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_DOC_USE_CNS_ENCODINGS:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            HPDF_UseCNSEncodings(pdf);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_DOC_USE_CNT_ENCODINGS:
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            HPDF_UseCNTEncodings(pdf);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_DOC_LOAD_PNG_IMAGE_FROM_FILE:
        {
            Tcl_DString ds;
            HPDF_Image image;
            const char *filename;
            TCLHPDF_ARGCHECK(3,3,2,"filename")

            filename = Tcl_UtfToExternalDString(NULL, 
                    Tcl_GetString(objv[2]), -1, &ds);

            image = HPDF_LoadPngImageFromFile(pdf, filename);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_IMAGE,
                        clientData, image)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_DOC_LOAD_PNG_IMAGE_FROM_FILE2:
        {
            Tcl_DString ds;
            HPDF_Image image;
            const char *filename;
            TCLHPDF_ARGCHECK(3,3,2,"filename")

            filename = Tcl_UtfToExternalDString(NULL, 
                    Tcl_GetString(objv[2]), -1, &ds);

            image = HPDF_LoadPngImageFromFile2(pdf, filename);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_IMAGE,
                        clientData, image)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_DOC_LOAD_JPEG_IMAGE_FROM_FILE:
        {
            Tcl_DString ds;
            HPDF_Image image;
            const char *filename;
            TCLHPDF_ARGCHECK(3,3,2,"filename")

            filename = Tcl_UtfToExternalDString(NULL, 
                    Tcl_GetString(objv[2]), -1, &ds);

            image = HPDF_LoadJpegImageFromFile(pdf, filename);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_IMAGE,
                        clientData, image)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }
 
        case TCLHPDF_DOC_LOAD_RAW_IMAGE_FROM_FILE:
        {
            Tcl_DString ds;
            HPDF_Image image;
            const char *filename;
            int colorspace, w, h;
            TCLHPDF_ARGCHECK(6,6,2,"filename width heifht colorspace")
            
            filename = Tcl_UtfToExternalDString(NULL, 
                    Tcl_GetString(objv[2]), -1, &ds);

            if (Tcl_GetIntFromObj(interp, objv[3], &w) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[4], &h) != TCL_OK) {
                return TCL_ERROR;
            }

            if (hpdf_getEnumFromStringObj (interp, 
                optionColorSpaceForRawString, 
                (int*)optionColorSpaceForRawEnum,
                objv[5], &colorspace) != TCL_OK) {
                return TCL_ERROR;
            }

            image = HPDF_LoadRawImageFromFile(pdf, filename, w, h, colorspace);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_IMAGE,
                        clientData, image)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_DOC_LOAD_RAW_IMAGE_FROM_MEM:
        {
            HPDF_Image image;
            const HPDF_BYTE *buf;
            int colorspace, w, h, bpc, buflen;
            TCLHPDF_ARGCHECK(7,7,2,"buf width height colorspace bits_per_component")

            buf = Tcl_GetByteArrayFromObj(objv[2], &buflen);

            if (Tcl_GetIntFromObj(interp, objv[3], &w) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[4], &h) != TCL_OK) {
                return TCL_ERROR;
            }
            if (hpdf_getEnumFromStringObj (interp, 
                optionColorSpaceForRawString, 
                (int*)optionColorSpaceForRawEnum,
                objv[5], &colorspace) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[6], &bpc) != TCL_OK) {
                return TCL_ERROR;
            }

            image = HPDF_LoadRawImageFromMem(pdf, buf, w, h, colorspace, bpc);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_IMAGE,
                        clientData, image)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_DOC_SET_INFO_ATTR:
        {
            Tcl_DString ds;
            int type;
            const char *value;
            TCLHPDF_ARGCHECK(4,4,2,"type value")

            if (hpdf_getEnumFromStringObj (interp, 
                optionInfoTypeSetString, 
                (int*)optionInfoTypeSetEnum,
                objv[2], &type) != TCL_OK) {
                return TCL_ERROR;
            }

            value = hpdf_TclObjToHpdfText(objv[3], &ds);

            HPDF_SetInfoAttr(pdf, type, value);
            Tcl_DStringFree(&ds);
            
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_GET_INFO_ATTR:
        {
            Tcl_DString ds;
            Tcl_Obj *result;
            int type;
            const char *value;
            TCLHPDF_ARGCHECK(3,3,2,"type")

            if (hpdf_getEnumFromStringObj (interp, 
                optionInfoTypeGetString, 
                (int*)optionInfoTypeGetEnum,
                objv[2], &type) != TCL_OK) {
                return TCL_ERROR;
            }

            value = HPDF_GetInfoAttr(pdf, type);
            result = Tcl_NewStringObj(
                    Tcl_ExternalToUtfDString(NULL, value, -1, &ds), -1);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            Tcl_SetObjResult(interp, result);
            return TCL_OK;
        }
        
        case TCLHPDF_DOC_SET_INFO_DATE_ATTR:
        {
            int type;
            HPDF_Date value;
            int year, month, day, hour, min, sec, tzhour, tzmin;
            char tzind;
            TCLHPDF_ARGCHECK(12,12,2,"type year month day hour min sec tzind tzhour tzmin")

            if (hpdf_getEnumFromStringObj (interp, 
                optionInfoTypeDateString, 
                (int*)optionInfoTypeDateEnum,
                objv[2], &type) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[3], &year) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[4], &month) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[5], &day) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[6], &hour) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[7], &min) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[8], &sec) != TCL_OK) {
                return TCL_ERROR;
            }
            tzind = Tcl_GetString(objv[9])[0];
            if (tzind == '+' || tzind == '-') {
                Tcl_SetObjResult(interp, Tcl_NewStringObj("tzind must be + or -",-1));
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[10], &tzhour) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[11], &tzmin) != TCL_OK) {
                return TCL_ERROR;
            }

            value.year        = year;
            value.month       = month;
            value.day         = day;
            value.hour        = hour;
            value.minutes     = min;
            value.seconds     = sec;
            value.ind         = tzind;
            value.off_hour    = tzhour;
            value.off_minutes = tzmin;
            HPDF_SetInfoDateAttr(pdf, type, value);

            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_SET_PASSWORD:
        {
            Tcl_DString ds1, ds2;
            const char *ownerpass, *userpass;
            TCLHPDF_ARGCHECK(3,4,2,"ownerpass ?userpass?")

            ownerpass = hpdf_TclObjToHpdfText(objv[2], &ds1);
            if (objc == 3) {
                userpass = NULL;
            } else {
                userpass = hpdf_TclObjToHpdfText(objv[3], &ds2);
            }

            HPDF_SetPassword(pdf, ownerpass, userpass);
            Tcl_DStringFree(&ds1);
            Tcl_DStringFree(&ds2);

            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_SET_PERMISSION:
        {
             int permission;
            TCLHPDF_ARGCHECK(3,3,2,"permission")

            if (hpdf_getFlagsFromListObj (interp, 
                optionPermissionString, 
                (int*)optionPermissionEnum,
                objv[2], &permission) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_SetPermission(pdf, permission);

            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_SET_ENCRYPTION_MODE:
        {
            int mode, key_len;
            TCLHPDF_ARGCHECK(4,4,2,"mode keylen")

            if (hpdf_getEnumFromStringObj (interp, 
                optionEncryptModeString, 
                (int*)optionEncryptModeEnum,
                objv[2], &mode) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[3], &key_len) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_SetEncryptionMode(pdf, mode, key_len);

            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_SET_COMPRESSION_MODE:
        {
            int mode;
            TCLHPDF_ARGCHECK(3,3,2,"mode")

            if (hpdf_getEnumFromStringObj (interp, 
                optionCompressionModeString, 
                (int*)optionCompressionModeEnum,
                objv[2], &mode) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_SetCompressionMode(pdf, mode);

            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_DOC_CREATE_EXT_GSTATE:
        {
            HPDF_ExtGState egstate;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            egstate = HPDF_CreateExtGState(pdf);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_EXTGSTATE,
                        clientData, egstate)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        default: break; /* doesnt reach here */
    }
    return TCL_OK;
}

