#include "tclhpdf.h"

int hpdf_EncoderObjCmd(ClientData clientData, Tcl_Interp *interp, 
        int objc, Tcl_Obj *const objv[])
{
    int index;
    HPDF_Encoder encoder = TCLHPDFOBJ_GET_ENCODER(clientData);
    HPDF_Doc pdf = TCLHPDFOBJ_GET_ROOTDOC(clientData);

    static CONST char *optionStrings[] = {
    "getType",
    "getByteType",
    "getUnicode",
    "getWritingMode",
     (char*)NULL
    };

    enum options {
    TCLHPDF_ENCODER_GETTYPE,
    TCLHPDF_ENCODER_GETBYTETYPE,
    TCLHPDF_ENCODER_GETUNICODE,
    TCLHPDF_ENCODER_GETWRITINGMODE
    };

    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
        return TCL_ERROR;
    }
    if (Tcl_GetIndexFromObj(interp, objv[1], optionStrings, 
                "option", 0, &index) != TCL_OK) {
        return TCL_ERROR;
    }

    switch (index) {
        case TCLHPDF_ENCODER_GETTYPE:
        {
            HPDF_EncoderType type;
            Tcl_Obj *typestring;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            type = HPDF_Encoder_GetType(encoder);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_getStringObjFromEnum (interp,
                optionEncoderTypeString, (int*)optionEncoderTypeEnum, 
                type, &typestring) != TCL_OK) {
                Tcl_AppendResult(interp, "encoder type isn't set or unknown.", NULL);
                return TCL_ERROR;
            }
            Tcl_SetObjResult(interp, typestring);
            return TCL_OK;
        }

        case TCLHPDF_ENCODER_GETBYTETYPE:
        {
            Tcl_Obj *typestring;
            HPDF_ByteType bytetype;
            char *text;
            int idx, len;
            TCLHPDF_ARGCHECK(4,4,2,"bytearraytext index")
            
            text = Tcl_GetByteArrayFromObj(objv[2], &len);
            
            if (Tcl_GetIntFromObj(interp, objv[3], &idx) != TCL_OK) {
                return TCL_ERROR;
            }
            bytetype = HPDF_Encoder_GetByteType(encoder, text, idx);
            hpdf_ErrorReturn(interp, pdf);
            
            if (hpdf_getStringObjFromEnum (interp,
                optionByteTypeString, (int*)optionByteTypeEnum,
                bytetype, &typestring) != TCL_OK) {
                Tcl_AppendResult(interp, "encoder type isn't set or unknown.", NULL);
                return TCL_ERROR;
            }
            
            Tcl_SetObjResult(interp, typestring);
            return TCL_OK;
        }

        case TCLHPDF_ENCODER_GETUNICODE:
        {
            int code;
            HPDF_UNICODE unicode;
            TCLHPDF_ARGCHECK(3,3,2,"code")
            
            if (Tcl_GetIntFromObj(interp, objv[2], &code) != TCL_OK) {
                return TCL_ERROR;
            }
            unicode = HPDF_Encoder_GetUnicode(encoder, code);
            hpdf_ErrorReturn(interp, pdf);

            Tcl_SetObjResult(interp, Tcl_NewIntObj(unicode));
            return TCL_OK;
        }

        case TCLHPDF_ENCODER_GETWRITINGMODE:
        {
            HPDF_WritingMode mode;
            Tcl_Obj *modestring;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            mode = HPDF_Encoder_GetWritingMode(encoder);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_getStringObjFromEnum (interp,
                optionWritingModeString, (int*)optionWritingModeEnum,
                mode, &modestring) != TCL_OK) {
                Tcl_AppendResult(interp, "encoder type isn't set or unknown.", NULL);
                return TCL_ERROR;
            }
            Tcl_SetObjResult(interp, modestring);
            return TCL_OK;
        }

        default: break;
    }
    return TCL_OK;
}
