#include "tclhpdf.h"

int hpdf_ExtGStateObjCmd(ClientData clientData, Tcl_Interp *interp, 
        int objc, Tcl_Obj *const objv[])
{
    int index;
    HPDF_ExtGState extgstate = TCLHPDFOBJ_GET_EXTGSTATE(clientData);
    HPDF_Doc pdf = TCLHPDFOBJ_GET_ROOTDOC(clientData);

    static CONST char *optionStrings[] = {
    "setAlphaStroke",
    "setAlphaFill",
    "setBlendMode",
     (char*)NULL
    };

    enum options {
    TCLHPDF_EXTGSTATE_SETALPHASTROKE,
    TCLHPDF_EXTGSTATE_SETALPHAFILL,
    TCLHPDF_EXTGSTATE_SETBLENDMODE
    };

    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
        return TCL_ERROR;
    }
    if (Tcl_GetIndexFromObj(interp, objv[1], optionStrings, 
                "option", 0, &index) != TCL_OK) {
        return TCL_ERROR;
    }

    switch (index) {
        case TCLHPDF_EXTGSTATE_SETALPHASTROKE:
        {
            double value;
            TCLHPDF_ARGCHECK(3,3,2,"value")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &value) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_ExtGState_SetAlphaStroke(extgstate, value);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_EXTGSTATE_SETALPHAFILL:
        {
            double value;
            TCLHPDF_ARGCHECK(3,3,2,"value")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &value) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_ExtGState_SetAlphaFill(extgstate, value);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_EXTGSTATE_SETBLENDMODE:
        {
            int mode;
            TCLHPDF_ARGCHECK(3,3,2,"mode")

            if (hpdf_getEnumFromStringObj (interp, 
                optionBlendModeString, (int*)optionBlendModeEnum,
                objv[2], &mode) != TCL_OK) {
                return TCL_ERROR;
            }

            HPDF_ExtGState_SetBlendMode(extgstate, mode);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        default: break;
    }
    return TCL_OK;
}
