#include "tclhpdf.h"

int hpdf_ImageObjCmd(ClientData clientData, Tcl_Interp *interp, 
        int objc, Tcl_Obj *const objv[])
{
    int index;
    HPDF_Image image = TCLHPDFOBJ_GET_IMAGE(clientData);
    HPDF_Doc pdf = TCLHPDFOBJ_GET_ROOTDOC(clientData);

    static CONST char *optionStrings[] = {
    "getSize",
    "getSize2",
    "getWidth",
    "getHeight",
    "getBitsPerComponent",
    "getColorSpace",
    "setColorMask",
    "setMaskImage",
     (char*)NULL
    };

    enum options {
    TCLHPDF_IMAGE_GETSIZE,
    TCLHPDF_IMAGE_GETSIZE2,
    TCLHPDF_IMAGE_GETWIDTH,
    TCLHPDF_IMAGE_GETHEIGHT,
    TCLHPDF_IMAGE_GETBITSPERCOMPONENT,
    TCLHPDF_IMAGE_GETCOLORSPACE,
    TCLHPDF_IMAGE_SETCOLORMASK,
    TCLHPDF_IMAGE_SETMASKIMAGE
    };

    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
        return TCL_ERROR;
    }
    if (Tcl_GetIndexFromObj(interp, objv[1], optionStrings, 
                "option", 0, &index) != TCL_OK) {
        return TCL_ERROR;
    }

    switch (index) {
        case TCLHPDF_IMAGE_GETSIZE:
        {
            HPDF_Point point;
            Tcl_Obj *list[2];
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            point = HPDF_Image_GetSize(image);
            hpdf_ErrorReturn(interp, pdf);

            list[0] = Tcl_NewDoubleObj(point.x);
            list[1] = Tcl_NewDoubleObj(point.y);
            Tcl_SetObjResult(interp, Tcl_NewListObj(2, list));
            return TCL_OK;
        }

        case TCLHPDF_IMAGE_GETSIZE2:
        {
            HPDF_Point size;
            TCLHPDF_ARGCHECK(3,3,2,"sizeName")
            HPDF_Image_GetSize2(image, &size);
            hpdf_ErrorReturn(interp, pdf);
            {
                Tcl_Obj *list[2];
                Tcl_Obj *ret;
                list[0] = Tcl_NewDoubleObj(size.x);
                list[1] = Tcl_NewDoubleObj(size.y);
                ret = Tcl_ObjSetVar2(interp, objv[2], NULL, 
                        Tcl_NewListObj(2, list), 0);
                if (ret == NULL) 
                    return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_IMAGE_GETWIDTH:
        {
            int n;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            n = HPDF_Image_GetWidth(image);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewIntObj(n));
            return TCL_OK;
        }

        case TCLHPDF_IMAGE_GETHEIGHT:
        {
            int n;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            n = HPDF_Image_GetHeight(image);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewIntObj(n));
            return TCL_OK;
        }

        case TCLHPDF_IMAGE_GETBITSPERCOMPONENT:
        {
            int n;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            n = HPDF_Image_GetBitsPerComponent(image);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewIntObj(n));
            return TCL_OK;
        }

        case TCLHPDF_IMAGE_GETCOLORSPACE:
        {
            const char *cs;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            cs = HPDF_Image_GetColorSpace(image);
            hpdf_ErrorReturn(interp, pdf);
            if (cs == NULL) {
                Tcl_AppendResult(interp, "unknown color space.", NULL);
                return TCL_ERROR;
            }
            Tcl_SetObjResult(interp, Tcl_NewStringObj(cs, -1));
            return TCL_OK;
        }

        case TCLHPDF_IMAGE_SETCOLORMASK:
        {
            int rmin, rmax, gmin, gmax, bmin, bmax;
            TCLHPDF_ARGCHECK(8,8,2,"rmin rmax gmin gmax bmin bmax")

            if (Tcl_GetIntFromObj(interp, objv[2], &rmin) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[3], &rmax) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[4], &gmin) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[5], &gmax) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[6], &bmin) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetIntFromObj(interp, objv[7], &bmax) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Image_SetColorMask (image, rmin, rmax, gmin, gmax, bmin, bmax);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_IMAGE_SETMASKIMAGE:
        {
            hpdf_Obj *obj;
            TCLHPDF_ARGCHECK(3,3,2,"hpdfImageObj")

            if (hpdf_GetObjFromCmdName (interp, objv[2], 
                        TCLHPDF_TYPE_IMAGE, &obj) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Image_SetMaskImage(image, TCLHPDFOBJ_GET_IMAGE(obj));
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        default: break;
    }
    return TCL_OK;
}
