#include "tclhpdf.h"

int hpdf_OutlineObjCmd(ClientData clientData, Tcl_Interp *interp, 
        int objc, Tcl_Obj *const objv[])
{
    int index;
    HPDF_Outline outline = TCLHPDFOBJ_GET_OUTLINE(clientData);
    HPDF_Doc pdf = TCLHPDFOBJ_GET_ROOTDOC(clientData);

    static CONST char *optionStrings[] = {
    "setOpened",
    "setDestination",
     (char*)NULL
    };

    enum options {
    TCLHPDF_OUTLINE_SETOPENED,
    TCLHPDF_OUTLINE_SETDESTINATION
    };

    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
        return TCL_ERROR;
    }
    if (Tcl_GetIndexFromObj(interp, objv[1], optionStrings, 
                "option", 0, &index) != TCL_OK) {
        return TCL_ERROR;
    }

    switch (index) {
        case TCLHPDF_OUTLINE_SETOPENED:
        {
            int opened;
            TCLHPDF_ARGCHECK(3,3,2,"opened")
            
            if (Tcl_GetBooleanFromObj(interp, objv[2], &opened) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Outline_SetOpened(outline, opened);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_OUTLINE_SETDESTINATION:
        {
            hpdf_Obj *obj;
            TCLHPDF_ARGCHECK(3,3,2,"hpdfDestinationObj")

            if (hpdf_GetObjFromCmdName (interp, objv[2], 
                        TCLHPDF_TYPE_DESTINATION, &obj) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Outline_SetDestination(outline, TCLHPDFOBJ_GET_DESTINATION(obj));
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }


        default: break;
    }
    return TCL_OK;
}
