#include "tclhpdf.h"

int hpdf_PageObjCmd(ClientData clientData, Tcl_Interp *interp, 
        int objc, Tcl_Obj *const objv[])
{
    int index;
    HPDF_STATUS ret;
    HPDF_Page page = TCLHPDFOBJ_GET_PAGE(clientData);
    HPDF_Doc pdf = TCLHPDFOBJ_GET_ROOTDOC(clientData);

    static CONST char *optionStrings[] = {
    "setWidth", "setHeight", "setSize", "setRotate", "createDestination",
    "createTextAnnot", "createLinkAnnot", "createURILinkAnnot",
    "textWidth", "measureText", "getWidth", "getHeight", "getGMode",
    "getCurrentPos", "getCurrentPos2", "getCurrentTextPos",
    "getCurrentTextPos2", "getCurrentFont", "getCurrentFontSize",
    "getTransMatrix", "getLineWidth", "getLineCap", "getLineJoin",
    "getMiterLimit", "getDash", "getFlat", "getCharSpace", "getWordSpace",
    "getHorizontalScalling", "getTextLeading", "getTextRenderingMode",
    "getTextRise", "getRGBFill", "getRGBStroke", "getCMYKFill",
    "getCMYKStroke", "getGrayFill", "getGrayStroke",
    "getStrokingColorSpace", "getFillingColorSpace", "getTextMatrix",
    "getGStateDepth", "setLineWidth", "setLineCap", "setLineJoin",
    "setMiterLimit", "setDash", "setFlat", "setExtGState", "gSave",
    "gRestore", "concat", "moveTo", "lineTo", "curveTo", "curveTo2",
    "curveTo3", "closePath", "rectangle", "stroke", "closePathStroke",
    "fill", "eofill", "fillStroke", "eofillStroke", "closePathFillStroke",
    "closePathEofillStroke", "endPath", "clip", "eoclip", "beginText",
    "endText", "setCharSpace", "setWordSpace", "setHorizontalScalling",
    "setTextLeading", "setFontAndSize", "setTextRenderingMode",
    "setTextRise", "moveTextPos", "moveTextPos2", "setTextMatrix",
    "moveToNextLine", "showText", "showTextNextLine", "showTextNextLineEx",
    "setGrayFill", "setGrayStroke", "setRGBFill", "setRGBStroke",
    "setCMYKFill", "setCMYKStroke", "executeXObject", "drawImage", "circle",
    "ellipse", "arc", "textOut", "textRect", "setSlideShow", (char*)NULL
    };

    enum options {
    TCLHPDF_PAGE_SETWIDTH,
    TCLHPDF_PAGE_SETHEIGHT,
    TCLHPDF_PAGE_SETSIZE,
    TCLHPDF_PAGE_SETROTATE,
    TCLHPDF_PAGE_CREATEDESTINATION,
    TCLHPDF_PAGE_CREATETEXTANNOT,
    TCLHPDF_PAGE_CREATELINKANNOT,
    TCLHPDF_PAGE_CREATEURILINKANNOT,
    TCLHPDF_PAGE_TEXTWIDTH,
    TCLHPDF_PAGE_MEASURETEXT,
    TCLHPDF_PAGE_GETWIDTH,
    TCLHPDF_PAGE_GETHEIGHT,
    TCLHPDF_PAGE_GETGMODE,
    TCLHPDF_PAGE_GETCURRENTPOS,
    TCLHPDF_PAGE_GETCURRENTPOS2,
    TCLHPDF_PAGE_GETCURRENTTEXTPOS,
    TCLHPDF_PAGE_GETCURRENTTEXTPOS2,
    TCLHPDF_PAGE_GETCURRENTFONT,
    TCLHPDF_PAGE_GETCURRENTFONTSIZE,
    TCLHPDF_PAGE_GETTRANSMATRIX,
    TCLHPDF_PAGE_GETLINEWIDTH,
    TCLHPDF_PAGE_GETLINECAP,
    TCLHPDF_PAGE_GETLINEJOIN,
    TCLHPDF_PAGE_GETMITERLIMIT,
    TCLHPDF_PAGE_GETDASH,
    TCLHPDF_PAGE_GETFLAT,
    TCLHPDF_PAGE_GETCHARSPACE,
    TCLHPDF_PAGE_GETWORDSPACE,
    TCLHPDF_PAGE_GETHORIZONTALSCALLING,
    TCLHPDF_PAGE_GETTEXTLEADING,
    TCLHPDF_PAGE_GETTEXTRENDERINGMODE,
    TCLHPDF_PAGE_GETTEXTRISE,
    TCLHPDF_PAGE_GETRGBFILL,
    TCLHPDF_PAGE_GETRGBSTROKE,
    TCLHPDF_PAGE_GETCMYKFILL,
    TCLHPDF_PAGE_GETCMYKSTROKE,
    TCLHPDF_PAGE_GETGRAYFILL,
    TCLHPDF_PAGE_GETGRAYSTROKE,
    TCLHPDF_PAGE_GETSTROKINGCOLORSPACE,
    TCLHPDF_PAGE_GETFILLINGCOLORSPACE,
    TCLHPDF_PAGE_GETTEXTMATRIX,
    TCLHPDF_PAGE_GETGSTATEDEPTH,
    TCLHPDF_PAGE_SETLINEWIDTH,
    TCLHPDF_PAGE_SETLINECAP,
    TCLHPDF_PAGE_SETLINEJOIN,
    TCLHPDF_PAGE_SETMITERLIMIT,
    TCLHPDF_PAGE_SETDASH,
    TCLHPDF_PAGE_SETFLAT,
    TCLHPDF_PAGE_SETEXTGSTATE,
    TCLHPDF_PAGE_GSAVE,
    TCLHPDF_PAGE_GRESTORE,
    TCLHPDF_PAGE_CONCAT,
    TCLHPDF_PAGE_MOVETO,
    TCLHPDF_PAGE_LINETO,
    TCLHPDF_PAGE_CURVETO,
    TCLHPDF_PAGE_CURVETO2,
    TCLHPDF_PAGE_CURVETO3,
    TCLHPDF_PAGE_CLOSEPATH,
    TCLHPDF_PAGE_RECTANGLE,
    TCLHPDF_PAGE_STROKE,
    TCLHPDF_PAGE_CLOSEPATHSTROKE,
    TCLHPDF_PAGE_FILL,
    TCLHPDF_PAGE_EOFILL,
    TCLHPDF_PAGE_FILLSTROKE,
    TCLHPDF_PAGE_EOFILLSTROKE,
    TCLHPDF_PAGE_CLOSEPATHFILLSTROKE,
    TCLHPDF_PAGE_CLOSEPATHEOFILLSTROKE,
    TCLHPDF_PAGE_ENDPATH,
    TCLHPDF_PAGE_CLIP,
    TCLHPDF_PAGE_EOCLIP,
    TCLHPDF_PAGE_BEGINTEXT,
    TCLHPDF_PAGE_ENDTEXT,
    TCLHPDF_PAGE_SETCHARSPACE,
    TCLHPDF_PAGE_SETWORDSPACE,
    TCLHPDF_PAGE_SETHORIZONTALSCALLING,
    TCLHPDF_PAGE_SETTEXTLEADING,
    TCLHPDF_PAGE_SETFONTANDSIZE,
    TCLHPDF_PAGE_SETTEXTRENDERINGMODE,
    TCLHPDF_PAGE_SETTEXTRISE,
    TCLHPDF_PAGE_MOVETEXTPOS,
    TCLHPDF_PAGE_MOVETEXTPOS2,
    TCLHPDF_PAGE_SETTEXTMATRIX,
    TCLHPDF_PAGE_MOVETONEXTLINE,
    TCLHPDF_PAGE_SHOWTEXT,
    TCLHPDF_PAGE_SHOWTEXTNEXTLINE,
    TCLHPDF_PAGE_SHOWTEXTNEXTLINEEX,
    TCLHPDF_PAGE_SETGRAYFILL,
    TCLHPDF_PAGE_SETGRAYSTROKE,
    TCLHPDF_PAGE_SETRGBFILL,
    TCLHPDF_PAGE_SETRGBSTROKE,
    TCLHPDF_PAGE_SETCMYKFILL,
    TCLHPDF_PAGE_SETCMYKSTROKE,
    TCLHPDF_PAGE_EXECUTEXOBJECT,
    TCLHPDF_PAGE_DRAWIMAGE,
    TCLHPDF_PAGE_CIRCLE,
    TCLHPDF_PAGE_ELLIPSE,
    TCLHPDF_PAGE_ARC,
    TCLHPDF_PAGE_TEXTOUT,
    TCLHPDF_PAGE_TEXTRECT,
    TCLHPDF_PAGE_SETSLIDESHOW
    };

    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
        return TCL_ERROR;
    }
    if (Tcl_GetIndexFromObj(interp, objv[1], optionStrings, 
                "option", 0, &index) != TCL_OK) {
        return TCL_ERROR;
    }

    switch (index) {
        case TCLHPDF_PAGE_SETWIDTH:
        {
            double value;
            TCLHPDF_ARGCHECK(3,3,2,"width")
            
            if (Tcl_GetDoubleFromObj(interp, objv[2], &value) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetWidth(page, value);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETHEIGHT:
        {
            double value;
            TCLHPDF_ARGCHECK(3,3,2,"height")
            
            if (Tcl_GetDoubleFromObj(interp, objv[2], &value) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetHeight(page, value);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETSIZE:
        {
            int size, direction;
            TCLHPDF_ARGCHECK(4,4,2,"size direction")

            if (hpdf_getEnumFromStringObj (interp, 
                optionPageSizesString, (int*)optionPageSizesEnum,
                objv[2], &size) != TCL_OK) {
                return TCL_ERROR;
            }
            if (hpdf_getEnumFromStringObj (interp, 
                optionPageDirectionString, (int*)optionPageDirectionEnum,
                objv[3], &direction) != TCL_OK) {
                return TCL_ERROR;
            }

            HPDF_Page_SetSize(page, size, direction);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETROTATE:
        {
            int angle;
            TCLHPDF_ARGCHECK(3,3,2,"angle")
            
            if (Tcl_GetIntFromObj(interp, objv[2], &angle) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetRotate(page, angle);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_CREATEDESTINATION:
        {
            HPDF_Destination dest;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            dest = HPDF_Page_CreateDestination(page);
            hpdf_ErrorReturn(interp, pdf);
            
            if (!dest) {
                Tcl_SetObjResult(interp, 
                    Tcl_NewStringObj("hpdf page failed to create destination",-1));
                return TCL_ERROR;
            }
            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_DESTINATION,
                        clientData, dest)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_PAGE_CREATETEXTANNOT:
        {
            HPDF_Annotation annot;
            HPDF_Rect rect;
            const char *text;
            HPDF_Encoder encoder;
            Tcl_DString ds;
            hpdf_Obj *encobj;
            TCLHPDF_ARGCHECK(4,5,2,"rect text ?encoder?")

            if (hpdf_TclListToHpdfRect(interp, "rect", objv[2], &rect) != TCL_OK) {
                return TCL_ERROR;
            }
            text = hpdf_TclObjToHpdfText(objv[3], &ds);
            
            if (objc == 4 || strlen(Tcl_GetString(objv[4])) == 0) {
                encoder = NULL;
            } else {
                if (hpdf_GetObjFromCmdName (interp, objv[4], 
                    TCLHPDF_TYPE_ENCODER, &encobj) != TCL_OK) {
                    Tcl_DStringFree(&ds);
                    return TCL_ERROR;
                }
                encoder = TCLHPDFOBJ_GET_ENCODER(encobj);
            }
            
            annot = HPDF_Page_CreateTextAnnot(page, rect, text, encoder);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            if (!annot) {
                Tcl_AppendResult(interp, 
                        "hpdf page failed to create text annotation", NULL);
                return TCL_ERROR;
            }
            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_ANNOTATION,
                        clientData, annot)!=TCL_OK) {
                return TCL_ERROR;
            }

            return TCL_OK;
        }

        case TCLHPDF_PAGE_CREATELINKANNOT:
        {
            HPDF_Annotation annot;
            HPDF_Rect rect;
            HPDF_Destination dest;
            hpdf_Obj *destobj;
            TCLHPDF_ARGCHECK(4,4,2,"rect hpdfDestinationObj")

            if (hpdf_TclListToHpdfRect(interp, "rect", objv[2], &rect) != TCL_OK) {
                return TCL_ERROR;
            }
            if (hpdf_GetObjFromCmdName (interp, objv[3], 
                TCLHPDF_TYPE_DESTINATION, &destobj) != TCL_OK) {
                return TCL_ERROR;
            }
            dest = TCLHPDFOBJ_GET_DESTINATION(destobj);

            annot = HPDF_Page_CreateLinkAnnot(page, rect, dest);
            hpdf_ErrorReturn(interp, pdf);

            if (!annot) {
                Tcl_AppendResult(interp, 
                        "hpdf page failed to create link annotation", NULL);
                return TCL_ERROR;
            }
            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_ANNOTATION,
                        clientData, annot)!=TCL_OK) {
                return TCL_ERROR;
            }

            return TCL_OK;
        }

        case TCLHPDF_PAGE_CREATEURILINKANNOT:
        {
            HPDF_Annotation annot;
            HPDF_Rect rect;
            const char *uri;
            Tcl_DString ds;
            TCLHPDF_ARGCHECK(4,4,2,"rect uri")

            if (hpdf_TclListToHpdfRect(interp, "rect", objv[2], &rect) != TCL_OK) {
                return TCL_ERROR;
            }
            uri = hpdf_TclObjToHpdfText(objv[3], &ds);

            annot = HPDF_Page_CreateURILinkAnnot(page, rect, uri);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            if (!annot) {
                Tcl_AppendResult(interp, 
                        "hpdf page failed to create uri annotation", NULL);
                return TCL_ERROR;
            }
            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_ANNOTATION,
                        clientData, annot)!=TCL_OK) {
                return TCL_ERROR;
            }

            return TCL_OK;
        }

        case TCLHPDF_PAGE_TEXTWIDTH:
        {
            Tcl_DString ds;
            const char *text;
            HPDF_REAL width;
            TCLHPDF_ARGCHECK(3,3,2,"text")

            text = hpdf_TclObjToHpdfText(objv[2], &ds);

            width = HPDF_Page_TextWidth(page, text);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(width));
            return TCL_OK;
        }
        
        case TCLHPDF_PAGE_MEASURETEXT:
        {
            Tcl_DString ds;
            const char *text;
            double width;
            int wordwrap, result;
            HPDF_REAL   realwidth;
            int blen;
            TCLHPDF_ARGCHECK(5,6,2,"text width wordwrap ?realwidthName?")

            text = hpdf_TclObjToHpdfText(objv[2], &ds);

            if (Tcl_GetDoubleFromObj(interp, objv[3], &width) != TCL_OK) {
                return TCL_ERROR;
            }

            if (Tcl_GetBooleanFromObj(interp, objv[4], &wordwrap) != TCL_OK) {
                return TCL_ERROR;
            }

            if (objc == 5) {
                blen = HPDF_Page_MeasureText(page, text, width, 
                        wordwrap, NULL);
            } else {
                blen = HPDF_Page_MeasureText(page, text, width, 
                        wordwrap, &realwidth);
            }
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);
            
            if (objc == 6) {
                if (Tcl_ObjSetVar2(interp, objv[5], NULL, 
                        Tcl_NewDoubleObj(realwidth), 0) == NULL) {
                    return TCL_ERROR;
                }
            }

            {
                Tcl_DString ds;
                Tcl_DStringInit(&ds);
                result = Tcl_GetCharLength(Tcl_NewStringObj(Tcl_ExternalToUtfDString(
                            NULL, text, blen, &ds),-1));
                Tcl_DStringFree(&ds);
            }
            Tcl_SetObjResult(interp, Tcl_NewIntObj(result));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETWIDTH:
        {
            HPDF_REAL val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetWidth(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETHEIGHT:
        {
            HPDF_REAL val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetHeight(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETGMODE:
        {
            int val;
            Tcl_Obj *ret;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetGMode(page);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_getStringObjFromEnum (interp,
                optionGraphicsModeString, (int*)optionGraphicsModeEnum, 
                val, &ret) != TCL_OK) {
                Tcl_AppendResult(interp, "unknown graphics mode.", NULL);
                return TCL_ERROR;
            }
            Tcl_SetObjResult(interp, ret);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETCURRENTPOS:
        {
            HPDF_Point pos;
            Tcl_Obj *list[2];
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            pos = HPDF_Page_GetCurrentPos(page);
            hpdf_ErrorReturn(interp, pdf);
            list[0] = Tcl_NewDoubleObj(pos.x);
            list[1] = Tcl_NewDoubleObj(pos.y);
            Tcl_SetObjResult(interp, Tcl_NewListObj(2, list));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETCURRENTPOS2:
        {
            HPDF_Point pos;
            Tcl_Obj *list[2];
            TCLHPDF_ARGCHECK(3,3,2,"posName")

            HPDF_Page_GetCurrentPos2(page, &pos);
            hpdf_ErrorReturn(interp, pdf);

            list[0] = Tcl_NewDoubleObj(pos.x);
            list[1] = Tcl_NewDoubleObj(pos.y);

            if (Tcl_ObjSetVar2(interp, objv[5], NULL, 
                    Tcl_NewListObj(2, list), 0) == NULL) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETCURRENTTEXTPOS:
        {
            HPDF_Point pos;
            Tcl_Obj *list[2];
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            pos = HPDF_Page_GetCurrentTextPos(page);
            hpdf_ErrorReturn(interp, pdf);
            list[0] = Tcl_NewDoubleObj(pos.x);
            list[1] = Tcl_NewDoubleObj(pos.y);
            Tcl_SetObjResult(interp, Tcl_NewListObj(2, list));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETCURRENTTEXTPOS2:
        {
            HPDF_Point pos;
            Tcl_Obj *list[2];
            TCLHPDF_ARGCHECK(3,3,2,"posName")

            HPDF_Page_GetCurrentTextPos2(page, &pos);
            hpdf_ErrorReturn(interp, pdf);

            list[0] = Tcl_NewDoubleObj(pos.x);
            list[1] = Tcl_NewDoubleObj(pos.y);

            if (Tcl_ObjSetVar2(interp, objv[5], NULL, 
                    Tcl_NewListObj(2, list), 0) == NULL) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETCURRENTFONT:
        {
            HPDF_Font font;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            font = HPDF_Page_GetCurrentFont(page);
            hpdf_ErrorReturn(interp, pdf);
            if (hpdf_NewHpdfCmd(interp, TCLHPDF_TYPE_FONT,
                        clientData, font)!=TCL_OK) {
                return TCL_ERROR;
            }
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETCURRENTFONTSIZE:
        {
            HPDF_REAL val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetCurrentFontSize(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETTRANSMATRIX:
        {
            Tcl_Obj *list[6];
            HPDF_TransMatrix matrix;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            matrix = HPDF_Page_GetTransMatrix(page);
            hpdf_ErrorReturn(interp, pdf);
            list[0] = Tcl_NewDoubleObj(matrix.a);
            list[1] = Tcl_NewDoubleObj(matrix.b);
            list[2] = Tcl_NewDoubleObj(matrix.c);
            list[3] = Tcl_NewDoubleObj(matrix.d);
            list[4] = Tcl_NewDoubleObj(matrix.x);
            list[5] = Tcl_NewDoubleObj(matrix.y);
            Tcl_SetObjResult(interp, Tcl_NewListObj(6, list));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETLINEWIDTH:
        {
            double val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetLineWidth(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETLINECAP:
        {
            double val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetLineCap(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETLINEJOIN:
        {
            HPDF_LineJoin join;
            Tcl_Obj *joinstring;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            join = HPDF_Page_GetLineJoin(page);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_getStringObjFromEnum (interp,
                optionLineJoinString, (int*)optionLineJoinEnum, 
                join, &joinstring) != TCL_OK) {
                Tcl_AppendResult(interp, "unknown line join type.", NULL);
                return TCL_ERROR;
            }
            Tcl_SetObjResult(interp, joinstring);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETMITERLIMIT:
        {
            double val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetMiterLimit(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETDASH:
        {
            int i;
            HPDF_DashMode dash;
            Tcl_Obj *list, *rlist[2];
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            dash = HPDF_Page_GetDash(page);
            hpdf_ErrorReturn(interp, pdf);
            
            list = Tcl_NewListObj(0, NULL);
            for (i=0; i<dash.num_ptn; i++) {
                Tcl_Obj *e = Tcl_NewIntObj(dash.ptn[i]);
                if (Tcl_ListObjAppendElement(interp, list, e) != TCL_OK) {
                    return TCL_ERROR;
                }
            }

            rlist[0] = list;
            rlist[1] = Tcl_NewIntObj(dash.phase);
            Tcl_SetObjResult(interp, Tcl_NewListObj(2, rlist));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETFLAT:
        {
            double val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetFlat(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETCHARSPACE:
        {
            double val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val= HPDF_Page_GetCharSpace(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETWORDSPACE:
        {
            double val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetWordSpace(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETHORIZONTALSCALLING:
        {
            double val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val= HPDF_Page_GetHorizontalScalling(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETTEXTLEADING:
        {
            double val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val= HPDF_Page_GetTextLeading(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETTEXTRENDERINGMODE:
        {
            HPDF_TextRenderingMode mode;
            Tcl_Obj *modestring;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            mode = HPDF_Page_GetTextRenderingMode(page);
            hpdf_ErrorReturn(interp, pdf);
            if (hpdf_getStringObjFromEnum (interp,
                optionTextRenderingModeString, (int*)optionTextRenderingModeEnum, 
                mode, &modestring) != TCL_OK) {
                Tcl_AppendResult(interp, "unknown text rendering mode.", NULL);
                return TCL_ERROR;
            }
            Tcl_SetObjResult(interp, modestring);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETTEXTRISE:
        {
            double val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetTextRise(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETRGBFILL:
        {
            Tcl_Obj *list[3];
            HPDF_RGBColor rgb;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            rgb = HPDF_Page_GetRGBFill(page);
            hpdf_ErrorReturn(interp, pdf);
            
            list[0] = Tcl_NewDoubleObj(rgb.r);
            list[1] = Tcl_NewDoubleObj(rgb.g);
            list[2] = Tcl_NewDoubleObj(rgb.b);
            Tcl_SetObjResult(interp, Tcl_NewListObj(3, list));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETRGBSTROKE:
        {
            Tcl_Obj *list[3];
            HPDF_RGBColor rgb;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            rgb = HPDF_Page_GetRGBStroke(page);
            hpdf_ErrorReturn(interp, pdf);
            
            list[0] = Tcl_NewDoubleObj(rgb.r);
            list[1] = Tcl_NewDoubleObj(rgb.g);
            list[2] = Tcl_NewDoubleObj(rgb.b);
            Tcl_SetObjResult(interp, Tcl_NewListObj(3, list));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETCMYKFILL:
        {
            Tcl_Obj *list[4];
            HPDF_CMYKColor cmyk;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            cmyk = HPDF_Page_GetCMYKFill(page);
            hpdf_ErrorReturn(interp, pdf);
            
            list[0] = Tcl_NewDoubleObj(cmyk.c);
            list[1] = Tcl_NewDoubleObj(cmyk.m);
            list[2] = Tcl_NewDoubleObj(cmyk.y);
            list[3] = Tcl_NewDoubleObj(cmyk.k);
            Tcl_SetObjResult(interp, Tcl_NewListObj(4, list));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETCMYKSTROKE:
        {
            Tcl_Obj *list[4];
            HPDF_CMYKColor cmyk;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            cmyk = HPDF_Page_GetCMYKStroke(page);
            hpdf_ErrorReturn(interp, pdf);
            
            list[0] = Tcl_NewDoubleObj(cmyk.c);
            list[1] = Tcl_NewDoubleObj(cmyk.m);
            list[2] = Tcl_NewDoubleObj(cmyk.y);
            list[3] = Tcl_NewDoubleObj(cmyk.k);
            Tcl_SetObjResult(interp, Tcl_NewListObj(4, list));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETGRAYFILL:
        {
            double val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetGrayFill(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETGRAYSTROKE:
        {
            double val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetGrayStroke(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewDoubleObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETSTROKINGCOLORSPACE:
        {
            HPDF_ColorSpace cs;
            Tcl_Obj *csstring;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            cs = HPDF_Page_GetStrokingColorSpace(page);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_getStringObjFromEnum (interp,
                optionColorSpaceString, (int*)optionColorSpaceEnum, 
                cs, &csstring) != TCL_OK) {
                Tcl_AppendResult(interp, "unknown color space.", NULL);
                return TCL_ERROR;
            }

            Tcl_SetObjResult(interp, csstring);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETFILLINGCOLORSPACE:
        {
            HPDF_ColorSpace cs;
            Tcl_Obj *csstring;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            cs = HPDF_Page_GetFillingColorSpace(page);
            hpdf_ErrorReturn(interp, pdf);

            if (hpdf_getStringObjFromEnum (interp,
                optionColorSpaceString, (int*)optionColorSpaceEnum, 
                cs, &csstring) != TCL_OK) {
                Tcl_AppendResult(interp, "unknown color space.", NULL);
                return TCL_ERROR;
            }

            Tcl_SetObjResult(interp, csstring);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETTEXTMATRIX:
        {
            Tcl_Obj *list[6];
            HPDF_TransMatrix matrix;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            matrix = HPDF_Page_GetTextMatrix(page);
            hpdf_ErrorReturn(interp, pdf);
            list[0] = Tcl_NewDoubleObj(matrix.a);
            list[1] = Tcl_NewDoubleObj(matrix.b);
            list[2] = Tcl_NewDoubleObj(matrix.c);
            list[3] = Tcl_NewDoubleObj(matrix.d);
            list[4] = Tcl_NewDoubleObj(matrix.x);
            list[5] = Tcl_NewDoubleObj(matrix.y);
            Tcl_SetObjResult(interp, Tcl_NewListObj(6, list));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GETGSTATEDEPTH:
        {
            int val;
            TCLHPDF_ARGCHECK(2,2,2,NULL)

            val = HPDF_Page_GetGStateDepth(page);
            hpdf_ErrorReturn(interp, pdf);
            Tcl_SetObjResult(interp, Tcl_NewIntObj(val));
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETLINEWIDTH:
        {
            double width;
            TCLHPDF_ARGCHECK(3,3,2,"width")
            
            if (Tcl_GetDoubleFromObj(interp, objv[2], &width) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetLineWidth(page, width);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETLINECAP:
        {
            int linecap;
            TCLHPDF_ARGCHECK(3,3,2,"linecap")

            if (hpdf_getEnumFromStringObj (interp, 
                optionLineCapString, (int*)optionLineCapEnum,
                objv[2], &linecap) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetLineCap(page, linecap);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETLINEJOIN:
        {
            int linejoin;
            TCLHPDF_ARGCHECK(3,3,2,"linejoin")

            if (hpdf_getEnumFromStringObj (interp, 
                optionLineJoinString, (int*)optionLineJoinEnum,
                objv[2], &linejoin) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetLineJoin(page, linejoin);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETMITERLIMIT:
        {
            double limit;
            TCLHPDF_ARGCHECK(3,3,2,"miterlimit")
            
            if (Tcl_GetDoubleFromObj(interp, objv[2], &limit) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetMiterLimit(page, limit);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETDASH:
        {
            int i, num, phase;
            HPDF_UINT16 dash[8];
            TCLHPDF_ARGCHECK(4,4,2,"dashlist phase")
            
            if (Tcl_ListObjLength(interp, objv[2], &num) != TCL_OK) {
                return TCL_ERROR;
            }
            if (num > 8) num = 8;

            for (i=0; i < num; i++) {
                Tcl_Obj *e;
                int di;
                if (Tcl_ListObjIndex(interp, objv[2], i, &e) != TCL_OK) {
                    return TCL_ERROR;
                }
                if (Tcl_GetIntFromObj(interp, e, &di) != TCL_OK) {
                    return TCL_ERROR;
                }
                dash[i] = di;
            }
            if (Tcl_GetIntFromObj(interp, objv[3], &phase) != TCL_OK) {
                return TCL_ERROR;
            }
            
            if (num == 0) {
                HPDF_Page_SetDash(page, NULL, 0, phase);
            } else {
                HPDF_Page_SetDash(page, dash, num, phase);
            }
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETFLAT:
        {
            double flatness;
            TCLHPDF_ARGCHECK(3,3,2,"flatness")
            
            if (Tcl_GetDoubleFromObj(interp, objv[2], &flatness) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetFlat(page, flatness);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETEXTGSTATE:
        {
            hpdf_Obj *obj;
            TCLHPDF_ARGCHECK(3,3,2,"hpdfExtGStateObj")

            if (hpdf_GetObjFromCmdName (interp, objv[2], 
                        TCLHPDF_TYPE_EXTGSTATE, &obj) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetExtGState(page, TCLHPDFOBJ_GET_EXTGSTATE(obj));
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_GSAVE:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_GSave(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_GRESTORE:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_GRestore(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_CONCAT:
        {
            double a,b,c,d,x,y;
            TCLHPDF_ARGCHECK(8,8,2,"a b c d x y")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &a) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &b) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &c) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &d) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[6], &x) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[7], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_Concat(page, a, b, c, d, x, y);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_MOVETO:
        {
            double x,y;
            TCLHPDF_ARGCHECK(4,4,2,"x y")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_MoveTo(page, x, y);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_LINETO:
        {
            double x,y;
            TCLHPDF_ARGCHECK(4,4,2,"x y")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_LineTo(page, x, y);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_CURVETO:
        {
            double x1,y1,x2,y2,x3,y3;
            TCLHPDF_ARGCHECK(8,8,2,"x1 y1 x2 y2 x3 y3")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &x1) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &y1) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &x2) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &y2) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[6], &x3) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[7], &y3) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_CurveTo(page, x1, y1, x2, y2, x3, y3);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_CURVETO2:
        {
            double x2,y2,x3,y3;
            TCLHPDF_ARGCHECK(6,6,2,"x2 y2 x3 y3")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &x2) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &y2) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &x3) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &y3) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_CurveTo2(page, x2, y2, x3, y3);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_CURVETO3:
        {
            double x1,y1,x3,y3;
            TCLHPDF_ARGCHECK(6,6,2,"x1 y1 x3 y3")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &x1) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &y1) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &x3) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &y3) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_CurveTo3(page, x1, y1, x3, y3);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_CLOSEPATH:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_ClosePath(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_RECTANGLE:
        {
            double x,y,w,h;
            TCLHPDF_ARGCHECK(6,6,2,"x y width height")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &w) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &h) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_Rectangle(page, x, y, w, h);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_STROKE:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_Stroke(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_CLOSEPATHSTROKE:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_ClosePathStroke(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_FILL:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_Fill(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_EOFILL:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_Eofill(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_FILLSTROKE:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_FillStroke(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_EOFILLSTROKE:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_EofillStroke(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_CLOSEPATHFILLSTROKE:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_ClosePathFillStroke(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_CLOSEPATHEOFILLSTROKE:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_ClosePathEofillStroke(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_ENDPATH:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_EndPath(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_CLIP:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_Clip(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_EOCLIP:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_Eoclip(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_BEGINTEXT:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_BeginText(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_ENDTEXT:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_EndText(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_SETCHARSPACE:
        {
            double value;
            TCLHPDF_ARGCHECK(3,3,2,"value")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &value) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetCharSpace(page, value);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETWORDSPACE:
        {
            double value;
            TCLHPDF_ARGCHECK(3,3,2,"value")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &value) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetWordSpace(page, value);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETHORIZONTALSCALLING:
        {
            double value;
            TCLHPDF_ARGCHECK(3,3,2,"value")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &value) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetHorizontalScalling(page, value);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETTEXTLEADING:
        {
            double value;
            TCLHPDF_ARGCHECK(3,3,2,"value")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &value) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetTextLeading(page, value);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETFONTANDSIZE:
        {
            hpdf_Obj *obj;
            double size;
            TCLHPDF_ARGCHECK(4,4,2,"hpdfFontObj size")

            if (hpdf_GetObjFromCmdName (interp, objv[2], 
                        TCLHPDF_TYPE_FONT, &obj) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &size) != TCL_OK) {
                return TCL_ERROR;
            }

            HPDF_Page_SetFontAndSize(page, TCLHPDFOBJ_GET_FONT(obj), size);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETTEXTRENDERINGMODE:
        {
            int mode;
            TCLHPDF_ARGCHECK(3,3,2,"mode")

            if (hpdf_getEnumFromStringObj (interp, 
                optionTextRenderingModeString, (int*)optionTextRenderingModeEnum,
                objv[2], &mode) != TCL_OK) {
                return TCL_ERROR;
            }

            HPDF_Page_SetTextRenderingMode(page, mode);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETTEXTRISE:
        {
            double value;
            TCLHPDF_ARGCHECK(3,3,2,"value")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &value) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetTextRise(page, value);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_MOVETEXTPOS:
        {
            double x,y;
            TCLHPDF_ARGCHECK(4,4,2,"x y")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_MoveTextPos(page, x, y);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_MOVETEXTPOS2:
        {
            double x,y;
            TCLHPDF_ARGCHECK(4,4,2,"x y")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_MoveTextPos2(page, x, y);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETTEXTMATRIX:
        {
            double a,b,c,d,x,y;
            TCLHPDF_ARGCHECK(8,8,2,"a b c d x y")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &a) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &b) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &c) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &d) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[6], &x) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[7], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetTextMatrix(page, a, b, c, d, x, y);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_MOVETONEXTLINE:
            TCLHPDF_ARGCHECK(2,2,2,NULL)
            HPDF_Page_MoveToNextLine(page);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;

        case TCLHPDF_PAGE_SHOWTEXT:
        {
            char *text;
            Tcl_DString ds;
            TCLHPDF_ARGCHECK(3,3,2,"text")
            
            text = hpdf_TclObjToHpdfText(objv[2], &ds);
            HPDF_Page_ShowText(page, text);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SHOWTEXTNEXTLINE:
        {
            char *text;
            Tcl_DString ds;
            TCLHPDF_ARGCHECK(3,3,2,"text")

            text = hpdf_TclObjToHpdfText(objv[2], &ds);
            HPDF_Page_ShowTextNextLine(page, text);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SHOWTEXTNEXTLINEEX:
        {
            double wordspace, charspace;
            char *text;
            Tcl_DString ds;
            TCLHPDF_ARGCHECK(5,5,2,"wordspace charspace text")
            
            if (Tcl_GetDoubleFromObj(interp, objv[2], &wordspace) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &charspace) != TCL_OK) {
                return TCL_ERROR;
            }
            text = hpdf_TclObjToHpdfText(objv[4], &ds);

            HPDF_Page_ShowTextNextLineEx(page, wordspace, charspace, text);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETGRAYFILL:
        {
            double gray;
            TCLHPDF_ARGCHECK(3,3,2,"gray")
            if (Tcl_GetDoubleFromObj(interp, objv[2], &gray) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetGrayFill(page, gray);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETGRAYSTROKE:
        {
            double gray;
            TCLHPDF_ARGCHECK(3,3,2,"gray")
            if (Tcl_GetDoubleFromObj(interp, objv[2], &gray) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetGrayStroke(page, gray);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETRGBFILL:
        {
            double r,g,b;
            TCLHPDF_ARGCHECK(5,5,2,"r g b")
            if (Tcl_GetDoubleFromObj(interp, objv[2], &r) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &g) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &b) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetRGBFill(page, r, g, b);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETRGBSTROKE:
        {
            double r,g,b;
            TCLHPDF_ARGCHECK(5,5,2,"r g b")
            if (Tcl_GetDoubleFromObj(interp, objv[2], &r) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &g) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &b) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetRGBStroke(page, r, g, b);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETCMYKFILL:
        {
            double c,m,y,k;
            TCLHPDF_ARGCHECK(6,6,2,"c m y k")
            if (Tcl_GetDoubleFromObj(interp, objv[2], &c) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &m) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &k) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetCMYKFill(page, c, m, y, k);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETCMYKSTROKE:
        {
            double c,m,y,k;
            TCLHPDF_ARGCHECK(6,6,2,"c m y k")
            if (Tcl_GetDoubleFromObj(interp, objv[2], &c) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &m) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &k) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetCMYKStroke(page, c, m, y, k);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_EXECUTEXOBJECT:
        {
            hpdf_Obj *obj;
            TCLHPDF_ARGCHECK(3,3,2,"xobject")

            if (hpdf_GetObjFromCmdName (interp, objv[2], 
                TCLHPDF_TYPE_IMAGE, &obj) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_ExecuteXObject(page, TCLHPDFOBJ_GET_IMAGE(obj));
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_DRAWIMAGE:
        {
            hpdf_Obj *obj;
            double x,y,w,h;
            TCLHPDF_ARGCHECK(7,7,2,"image x y width height")

            if (hpdf_GetObjFromCmdName (interp, objv[2], 
                TCLHPDF_TYPE_IMAGE, &obj) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &x) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &w) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[6], &h) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_DrawImage(page, TCLHPDFOBJ_GET_IMAGE(obj), x, y, w, h);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_CIRCLE:
        {
            double x,y,ray;
            TCLHPDF_ARGCHECK(5,5,2,"x y ray")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &ray) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_Circle(page, x, y, ray);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_ELLIPSE:
        {
            double x,y,xray,yray;
            TCLHPDF_ARGCHECK(6,6,2,"x y xray yray")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &xray) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &yray) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_Ellipse(page, x, y, xray, yray);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_ARC:
        {
            double x,y,ray,ang1,ang2;
            TCLHPDF_ARGCHECK(7,7,2,"x y ray ang1 ang2")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &x) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &y) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &ray) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &ang1) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[6], &ang2) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_Arc(page, x, y, ray, ang1, ang2);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_TEXTOUT:
        {
            Tcl_DString ds;
            double xpos,ypos;
            char *text;
            TCLHPDF_ARGCHECK(5,5,2,"xpos ypos text")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &xpos) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &ypos) != TCL_OK) {
                return TCL_ERROR;
            }
            text = hpdf_TclObjToHpdfText(objv[4], &ds);

            HPDF_Page_TextOut(page, xpos, ypos, text);
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        case TCLHPDF_PAGE_TEXTRECT:
        {
            Tcl_DString ds;
            double left,top,right,bottom;
            char *text;
            int align,len,textlen=0;
            TCLHPDF_ARGCHECK(8,9,2,"left top right bottom text align ?len?")

            if (Tcl_GetDoubleFromObj(interp, objv[2], &left) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &top) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &right) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[5], &bottom) != TCL_OK) {
                return TCL_ERROR;
            }
            text = hpdf_TclObjToHpdfText(objv[6], &ds);
            
            if (hpdf_getEnumFromStringObj (interp, 
                optionTextAlignmentString, (int*)optionTextAlignmentEnum,
                objv[7], &align) != TCL_OK) {
                return TCL_ERROR;
            }
            if (objc == 8) {
                HPDF_Page_TextRect(page, left, top, right, bottom,
                        text, align, NULL);
            } else {
                Tcl_DString ds2;
                Tcl_DStringInit(&ds2);
                HPDF_Page_TextRect(page, left, top, right, bottom,
                        text, align, &len);
                textlen = Tcl_GetCharLength(Tcl_NewStringObj(
                            Tcl_ExternalToUtfDString(NULL, text, len, &ds2),-1));
                Tcl_DStringFree(&ds2);
            }
            Tcl_DStringFree(&ds);
            hpdf_ErrorReturn(interp, pdf);

            if (objc != 8) {
                if (Tcl_ObjSetVar2(interp, objv[8], NULL, 
                        Tcl_NewIntObj(textlen), 0) == NULL) {
                    return TCL_ERROR;
                }
            }
            return TCL_OK;
        }

        case TCLHPDF_PAGE_SETSLIDESHOW:
        {
            int type;
            double disptime, transtime;
            TCLHPDF_ARGCHECK(5,5,2,"type disptime transtime")

            if (hpdf_getEnumFromStringObj (interp, 
                optionTransitionStyleString, (int*)optionTransitionStyleEnum,
                objv[2], &type) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[3], &disptime) != TCL_OK) {
                return TCL_ERROR;
            }
            if (Tcl_GetDoubleFromObj(interp, objv[4], &transtime) != TCL_OK) {
                return TCL_ERROR;
            }
            HPDF_Page_SetSlideShow(page, type, disptime, transtime);
            hpdf_ErrorReturn(interp, pdf);
            return TCL_OK;
        }

        default: break;
    }
    return TCL_OK;
}
