#include "tclhpdf.h"

int hpdf_TclListToHpdfRect (Tcl_Interp *interp, const char *name, Tcl_Obj *list, HPDF_Rect *rect) {
    int len;
    double val;
    Tcl_Obj *obj;

    if (Tcl_ListObjLength(interp, list, &len) != TCL_OK) {
        return TCL_ERROR;
    }
    if (len != 4) {
        Tcl_AppendResult(interp, name, " must be {left bottom right top} list.", NULL);
        return TCL_ERROR;
    }
    
    while (len-- > 0) {
        if (Tcl_ListObjIndex(interp, list, len, &obj) != TCL_OK) {
            return TCL_ERROR;
        }
        if (Tcl_GetDoubleFromObj(interp, obj, &val) != TCL_OK) {
            return TCL_ERROR;
        }
        switch (len) {
            case 3:rect->top    = val; break;
            case 2:rect->right  = val; break;
            case 1:rect->bottom = val; break;
            case 0:rect->left   = val; break;
        }
    }
    return TCL_OK;
}

char * hpdf_TclObjToHpdfText (Tcl_Obj *obj, Tcl_DString *ds) {
    int len;
    char *text;
    
    if (obj->typePtr != NULL && strcmp(obj->typePtr->name, "bytearray") == 0) {
        Tcl_DStringInit(ds);
        text = Tcl_GetByteArrayFromObj(obj, &len);
        text = Tcl_DStringAppend(ds, text, len);
        text = Tcl_DStringAppend(ds, "\0" ,1);
    } else {
        text = Tcl_UtfToExternalDString(NULL,
                Tcl_GetString(obj), -1, ds);
    }
    return text;
}
